###############################################################################
# Backup.lib.pl                                                               #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################
package Backup;

use strict;
use Fcntl;
use File::Copy;
use Benchmark;

###############################################################################
# Backup function
sub Backup {
  my %in = (
    TO     => "",
    TABLE  => "",
    NUMBER => "",
    CONFIG => [],
    FILE   => [],
    EXPORT =>  1,
    @_
  );

  my $BackupDir = $in{'TO'};

  my $tmp = join ("\|", @{ $in{'FILE'} });
  my $RE = eval "sub { m/\\.($tmp)/i }";

  print "- - ".$in{'NUMBER'}.")Copying the '$in{'TABLE'}' directory...\n";

    print "- - - Creating a backup directory for the '$in{'TABLE'}' data... ";
    mkdir ("$BackupDir/$in{'TABLE'}", $SYSTEM::PERMISSION) || (print "Error: $!\n" and return);
    chmod (0777, "$BackupDir/$in{'TABLE'}");
    print "Done.\n";

    if (scalar ( @{ $in{'FILE'} } ) > 0) {
      print "- - - Opening the '$in{'TABLE'}' directory...\n";
      opendir (DIR, "$SYSTEM::DB_PATH/$in{'TABLE'}") || (print "Error: $!\n" and return -5);
        while ( defined (my $FILE = readdir (DIR)) ) {
          next if ($FILE =~ /^\.\.?$/);
          unless (-d $FILE) {
            $_ = $FILE;
            print "- - - - Reading the file '$FILE'... ";
            if (&{$RE}) {
              print "copying... ";
              copy ("$SYSTEM::DB_PATH/$in{'TABLE'}/$FILE", "$BackupDir/$in{'TABLE'}/$FILE") || print "error: $!... ";
              print "Done.\n";
            } else {
              print "skipped\n";
            }
          }
        }
      closedir (DIR);
      print "- - - Done.\n";
    }

    if ($in{'EXPORT'} == 1) {
      print "- - - Exporting the '$in{'TABLE'}' database... ";
      if (my $Export = &ExportDatabase($in{'TABLE'})) {
        if (open (EXPORT, ">$BackupDir/$in{'TABLE'}/$in{'TABLE'}.export")) {
            #chmod (0766, "$BackupDir/$in{'TABLE'}/$in{'TABLE'}.export");
            print EXPORT $Export;
          close (EXPORT);
          print "Done.\n";
        } else {
          print "Error: $!\n";
        }
      } else {
        return;
      }
    }

    foreach my $Config ( @{ $in{'CONFIG'} } ) {
      print "- - - Copying the config file of '$in{'TABLE'}' database... ";
      copy ("$SYSTEM::DB_PATH/$Config", "$BackupDir/$Config") || print "Error: $!\n";
      print "Done.\n";
    }

  print "- - Done.\n\n";

  return 1;
}

###############################################################################
# All function
sub All {
  my %in = (
    DETAIL     =>  1,
    @_
  );

  my $T0   = new Benchmark;
  my $TIME = time;

  print "<!--\n" if $in{'DETAIL'} != 1;

  print "- Begin backing up the ultraboard 2000.\n";

  print "- Creating a directory for backup '$SYSTEM::BACKUP/UB2K-$TIME'... ";
  my $BackupDir = "$SYSTEM::BACKUP/UB2K-$TIME";
  mkdir ($BackupDir, $SYSTEM::PERMISSION) || (print "Error: $!\n\n" and return -5);
  chmod (0777, $BackupDir);
  print "Done.\n\n";

  print "- Backing up the member groups data...\n\n";

    my $Counter = 0;

    ###########################################################################
    # Copying the member groups
    &Backup (
      TO     => $BackupDir,
      TABLE  => "Groups",
      NUMBER => ++$Counter,
      CONFIG => ["Groups.cfg"],
      FILE   => ["counter"]
    );

    ###########################################################################
    # Copying the member accounts
    &Backup (
      TO     => $BackupDir,
      TABLE  => "Accounts",
      NUMBER => ++$Counter,
      CONFIG => ["Accounts.cfg", "accounts.username", "accounts.nickname", "accounts.email"],
      FILE   => ["counter"]
    );

    ###########################################################################
    # Copying the member profiles
    &Backup (
      TO     => $BackupDir,
      TABLE  => "Profiles",
      NUMBER => ++$Counter,
      CONFIG => ["Profiles.cfg"],
      FILE   => ["counter"]
    );

    ###########################################################################
    # Copying the member sessions
    &Backup (
      TO     => $BackupDir,
      TABLE  => "Sessions",
      NUMBER => ++$Counter,
      CONFIG => ["Sessions.cfg"],
      FILE   => ["counter"],
      EXPORT => 0
    );

    ###########################################################################
    # Copying the password
    &Backup (
      TO     => $BackupDir,
      TABLE  => "Password",
      NUMBER => ++$Counter,
      CONFIG => ["Password.cfg"],
      FILE   => ["counter"],
      EXPORT => 0
    );

    ###########################################################################
    # Copying the categories
    &Backup (
      TO     => $BackupDir,
      TABLE  => "Categories",
      NUMBER => ++$Counter,
      CONFIG => ["Categories.cfg"],
      FILE   => ["counter", "dat"]
    );

    ###########################################################################
    # Copying the boards
    &Backup (
      TO     => $BackupDir,
      TABLE  => "Boards",
      NUMBER => ++$Counter,
      CONFIG => ["Boards.cfg"],
      FILE   => ["counter"]
    );

    ###########################################################################
    # Copying the board
    print "- - Getting the boards information... ";
    (
      my ($Boards) = &DB::Query (
        TABLE   => "Boards",
        COLUMNS => ["ID"],
      )
    ) || (print "Error: $DB::Error\n" and return -5);
    print "Done.\n";

    for (my $index = 0; $index <= $#{ $Boards }; $index++) {
      my $BID = $Boards->[$index]{'ID'};

      &Backup (
        TO     => $BackupDir,
        TABLE  => "B$BID",
        NUMBER => ++$Counter,
        CONFIG => ["B$BID.cfg"],
        FILE   => ["counter", "inc", "dat", "src", "post", "html"]
      );
    }

  my $T1 = new Benchmark;

  print "- Done. Used".timestr( timediff($T1, $T0) )."\n";

  print "-->\n"  if $in{'DETAIL'} != 1;

  return 1;
}

sub ExportDatabase {
  my ($Table) = @_;

  my $ExportData;

  &Standard::Require ("$Table.cfg", "$SYSTEM::DB_PATH/$Table.cfg");
  &Standard::lock    ("$Table.lock");

  my $DBTable     = $DB::Table      { $Table };
  my $PrimaryKey  = $DB::PrimaryKey { $Table };
  my $Columns     = $DB::Columns    { $Table };

  my %DB = ();
  tie (my %DBM, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || (print "Error: $!\n" and return);
    foreach my $KEY (keys %DBM) {
      $DB{$KEY} = $DBM{$KEY};
    }
  untie (%DBM);

  $ExportData .= join ("|", @{$Columns})."\n";

  if ($DB::Table{ $Table }->{ $DB::PrimaryKey{ $Table } }[1] eq "string") {
    foreach my $KEY ( sort { $a cmp $b } keys %DB ) {
      $ExportData .= $KEY."|~|".$DB{$KEY}."\n";
    }
  } else {
    foreach my $KEY ( sort { $a <=> $b } keys %DB ) {
      $ExportData .= $KEY."|~|".$DB{$KEY}."\n";
    }
  }

  &Standard::unLock("$Table.lock");

  return $ExportData;
}

###############################################################################
1; # end of Backup.lib.pl file
###############################################################################