###############################################################################
# Compile.lib.pl                                                              #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################
package Compile;

use strict;
use File::Copy;
use Benchmark;

###############################################################################
# All function
sub All {
  my %in = (
    THEME      => "",
    DETAIL     =>  1,
    @_
  );

  $in{'THEME'} = $SYSTEM::DEFAULT_THEME if $in{'THEME'} eq "";

  my $T0   = new Benchmark;

  print "<!--\n" if $in{'DETAIL'} != 1;

  &Standard::Require($in{'THEME'}.".cfg", $SYSTEM::THEME_PATH."/".$in{'THEME'}."/".$in{'THEME'}.".cfg");

  print "- Begin to move all '$in{'THEME'}' images to images directory.\n";
  mkdir ($SYSTEM::IMAGE_PATH."/".$in{'THEME'}, $SYSTEM::PERMISSION);
  chmod (0777, $SYSTEM::IMAGE_PATH."/".$in{'THEME'});

  opendir (DIR, "$SYSTEM::THEME_PATH/".$in{'THEME'}."/Images");
    while ( defined (my $FILE = readdir (DIR)) ) {
      next if ($FILE =~ /^\.\.?$/);
      unless (-d $FILE) {
        print "- - Copying $FILE to images directory... ";
        if (copy ("$SYSTEM::THEME_PATH/".$in{'THEME'}."/Images/".$FILE, $SYSTEM::IMAGE_PATH."/".$in{'THEME'}."/".$FILE)) {
          print "Done.\n";
        } else {
          print "Fail.\n";
        }
      }
    }
  closedir (DIR);
  print "- Done.\n\n";

  print "- Begin to compile the theme '$in{'THEME'}'.\n";

  my $ThemeDir = $UB::INTERFACES_PATH."/".$in{'THEME'};
  unless (-e $ThemeDir) {
    print "- Creating a directory for the theme '$ThemeDir'... ";
    mkdir ($ThemeDir, $SYSTEM::PERMISSION) || (print "Error: $!\n\n" and return -5);
    chmod (0777, $ThemeDir);

    print "Done.\n\n";
  }

  $Compile::LayoutPath   = "$SYSTEM::THEME_PATH/$in{'THEME'}/Layout";
  $Compile::LanguagePath = "$SYSTEM::THEME_PATH/$in{'THEME'}/Language";

  print "- Compiling the theme...\n\n";

  my $Counter = 0;
  foreach my $NAME (keys %THEME::LAYOUT) {
    $Counter++;
    print "- - $Counter) $NAME\n";
    if (
      &Compile (
        LAYOUT   => $THEME::LAYOUT  {$NAME},
        LANGUAGE => $THEME::LANGUAGE{$NAME},
        TO       => $ThemeDir
      )
    ) {
      print "- - Done.\n\n";
    }
  }

  my $T1 = new Benchmark;

  print "- Done. Used ".timestr( timediff($T1, $T0) )."\n";

  print "-->\n"  if $in{'DETAIL'} != 1;

  return 1;
}

###############################################################################
# Layout function
sub Layout {
  my %in = (
    THEME      => "",
    LAYOUT     => "",
    DETAIL     =>  1,
    @_
  );

  $in{'THEME'} = $SYSTEM::DEFAULT_THEME if $in{'THEME'} eq "";

  my $T0   = new Benchmark;

  print "<!--\n" if $in{'DETAIL'} != 1;

  &Standard::Require($in{'THEME'}.".cfg", $SYSTEM::THEME_PATH."/".$in{'THEME'}."/".$in{'THEME'}.".cfg");

  print "- Begin to compile the theme '$in{'THEME'}'.\n";

  my $ThemeDir = $UB::INTERFACES_PATH."/".$in{'THEME'};
  unless (-e $ThemeDir) {
    print "- Creating a directory for the theme '$ThemeDir'... ";
    mkdir ($ThemeDir, $SYSTEM::PERMISSION) || (print "Error: $!\n\n" and return -5);
    chmod (0777, $ThemeDir);

    print "Done.\n\n";
  }

  $Compile::LayoutPath   = "$SYSTEM::THEME_PATH/Layouts/$in{'THEME'}";
  $Compile::LanguagePath = "$SYSTEM::THEME_PATH/Languages/$in{'THEME'}";

  print "- Compiling the theme...\n\n";

  print "- - $in{'LAYOUT'}\n";
  if (
    &Compile (
      LAYOUT   => $THEME::LAYOUT  {$in{'LAYOUT'}},
      LANGUAGE => $THEME::LANGUAGE{$in{'LAYOUT'}},
      TO       => $ThemeDir
    )
  ) {
    print "- - Done.\n\n";
  }

  my $T1 = new Benchmark;

  print "- Done. Used ".timestr( timediff($T1, $T0) )."\n";

  print "-->\n"  if $in{'DETAIL'} != 1;

  return 1;
}

###############################################################################
# Compile function
sub Compile {
  my %in = @_;

  unless ( ($in{'LAYOUT'}) && (-e $Compile::LayoutPath."/".$in{'LAYOUT'}) ) {
    require $Compile::LayoutPath."/".$in{'LAYOUT'} || (print "- - Error: $!\n" and return);
  }

  unless ( ($in{'TO'}) && (-e $in{'TO'}) ) {
    print "- - Error: missing to value.\n";
    return;
  }

  if ( ($in{'LANGUAGE'}) && (-e $Compile::LanguagePath."/".$in{'LANGUAGE'}) ) {
    require $Compile::LanguagePath."/".$in{'LANGUAGE'} || (print "- - Error: $!\n" and return);
  }

  my ($LAYOUT_FILE) = $in{'LAYOUT'};

  $LAYOUT_FILE =~ /^([\w\-\.]+)\.([a-zA-Z0-9]+)$/;

  my $LAYOUT_FILENAME = $1;
  my $LAYOUT_FILE_EXT = $2;

  print "- - - Reading layout '$LAYOUT_FILE'... ";
  chmod (0766, $Compile::LayoutPath."/".$LAYOUT_FILE);
  open (LAYOUT, $Compile::LayoutPath."/".$LAYOUT_FILE) || (print "Error: $!\n" and return);
    my $LAYOUT = join ("", <LAYOUT>);
  close (LAYOUT);
  print "Done.\n";

  print "- - - Preparing the variable... ";
  my %VARIABLE = (
    "GENERAL.SITE.NAME"               => $GENERAL::SITE_NAME                          ,
    "GENERAL.SITE.URL"                => $GENERAL::SITE_URL                           ,
    "GENERAL.ULTRABOARD.NAME"         => $GENERAL::ULTRABOARD_NAME                    ,
    "GENERAL.ULTRABOARD.URL"          => $SYSTEM::UB_URL."/UltraBoard.".$SYSTEM::EXT  ,
    "GENERAL.ULTRABOARD.DESCRIPTION"  => $GENERAL::ULTRABOARD_DESCRIPTION             ,
    "GENERAL.IMAGES.URL"              => $SYSTEM::IMAGE_URL."/".$SYSTEM::DEFAULT_THEME,
    "GENERAL.UPLOAD.URL"              => $SYSTEM::UPLOAD_URL                          ,
    "GENERAL.CP.URL"                  => $SYSTEM::CP_URL."/Panel.".$SYSTEM::EXT       ,
    "GENERAL.OTHERS.URL"              => $SYSTEM::UB_URL."/Others.".$SYSTEM::EXT      ,
    "GENERAL.CONTACT_EMAIL"           => $GENERAL::CONTACT_EMAIL                      ,
    "GENERAL.TIMEZONE"                => $SYSTEM::TIMEZONE                            ,
    "GENERAL.SHOW_IP_HOST"            => $GENERAL::SHOW_IP_HOST                       ,

    "GENERAL.THEME"                   => $SYSTEM::DEFAULT_THEME                       ,

    "GENERAL.CATEGORY_ONLY"           => $GENERAL::CATEGORY_ONLY                      ,

    "GENERAL.PRINT"                   => $GENERAL::PRINT                              ,
    "GENERAL.FORWARD"                 => $GENERAL::FORWARD                            ,

    "GENERAL.DISPLAY_AGREEMENT"       => $GENERAL::DISPLAY_AGREEMENT                  ,
    "GENERAL.AGREEMENT"               => $GENERAL::AGREEMENT                          ,

    "GENERAL.VERIFY_EMAIL"            => $GENERAL::VERIFY_EMAIL                       ,
    "GENERAL.REVIEW_ACCOUNT"          => $GENERAL::REVIEW_ACCOUNT                     ,

    "GENERAL.NICKNAME_DUPLICATE"      => $GENERAL::NICKNAME_DUPLICATE                 ,
    "GENERAL.EMAIL_DUPLICATE"         => $GENERAL::EMAIL_DUPLICATE                    ,

    "GENERAL.NOTIFY_MEMBER_POST"      => $GENERAL::NOTIFY_MEMBER_POST                 ,

    "GENERAL.MAX_SUBJECT_LENGTH"      => $GENERAL::MAX_SUBJECT_LENGTH                 ,
    "GENERAL.MAX_MESSAGE_LENGTH"      => $GENERAL::MAX_MESSAGE_LENGTH                 ,

    "GENERAL.MAX_PHOTO_LENGTH"        => $GENERAL::MAX_PHOTO_LENGTH                   ,
    "GENERAL.MAX_PHOTO_X"             => $GENERAL::MAX_PHOTO_X                        ,
    "GENERAL.MAX_PHOTO_Y"             => $GENERAL::MAX_PHOTO_Y                        ,

    %LANGUAGE::STATIC
  );

  foreach my $KEY (keys %UB::STYLE) {
    $VARIABLE{ "STYLE.".$KEY } = $UB::STYLE{ $KEY };
  }

  foreach my $KEY (keys %VARIABLE) {
    $VARIABLE{$KEY} =~ s/\\/\\\\/g;
    $VARIABLE{$KEY} =~ s/\"/\\\"/g;
    $VARIABLE{$KEY} =~ s/\$/\\\$/g;
    $VARIABLE{$KEY} =~ s/\@/\\\@/g;
    $VARIABLE{$KEY} =~ s/\%/\\\%/g;
  }
  print "Done.\n";

  print "- - - Parsing static tags...\n";
  my $TAG_START = quotemeta("<:");
  my $TAG_END   = quotemeta(":>");

    print "- - - - Parsing the include tags...\n";
    $LAYOUT =~ s/$TAG_START\s*INCLUDE\s+\"(.+)\"\s*$TAG_END/
      if (-e $Compile::LayoutPath.'\/'.$THEME::INCLUDE{$1}) {
        print "- - - - - Opening the include file '$Compile::LayoutPath\/$THEME::INCLUDE{$1}'... ";
        chmod (0766, $Compile::LayoutPath.'\/'.$THEME::INCLUDE{$1});
        open (INCLUDE, $Compile::LayoutPath.'\/'.$THEME::INCLUDE{$1}) || print "Error: $!... ";
          my $INCLUDE = join ("", <INCLUDE>);
        close (INCLUDE);
        print "Done.\n";

        if (-e $Compile::LanguagePath.'\/'.$THEME::LANGUAGE{"INCLUDE_".$1}) {
          print "- - - - - Opening the language package of include file '$Compile::LanguagePath\/".$THEME::LANGUAGE{"INCLUDE_".$1}."'... ";
          chmod (0766, $Compile::LanguagePath."\/".$THEME::LANGUAGE{"INCLUDE_".$1});
          do ($Compile::LanguagePath."\/".$THEME::LANGUAGE{"INCLUDE_".$1}) || print "Error: $!... ";
          %VARIABLE = (%VARIABLE, %LANGUAGE::STATIC);
          print "Done.\n";
        }

        $INCLUDE;
      }
    /gie;
    print "- - - - Done.\n";
  $LAYOUT =~ s/\\/\\\\/g;
  $LAYOUT =~ s/\"/\\\"/g;
  $LAYOUT =~ s/\$/\\\$/g;
  $LAYOUT =~ s/\@/\\\@/g;
  $LAYOUT =~ s/\%/\\\%/g;
  $LAYOUT =~ s/\|/\&\#124\;/g;

    print "- - - - Parsing the if/unless w/== statements...\n";
    while (
      $LAYOUT =~ s/$TAG_START\s*([\w-]+)\s*:\s*(IF|UNLESS)\s*(\S+)\s+==\s+\"(.+)\"\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*\2\s*$TAG_END/
        if ($2 eq "UNLESS") {
          print "- - - - - Parsing the unless \$VARIABLE{'$3'} == '$4' statement... ";
          if ( $VARIABLE{$3} ne $4 ) {
            print "True.\n";
            $5;
          } else {
            print "False.\n";
            ""
          }
        } else {
          print "- - - - - Parsing the if \$VARIABLE{'$3'} == '$4' statement... ";
          if ( $VARIABLE{$3} eq $4 ) {
            print "True.\n";
            $5;
          } else {
            print "False.\n";
            ""
          }
        }
      /ige
    ) {

    }
    print "- - - - Done.\n";

    print "- - - - Parsing the if/unless statements...\n";
    while (
      $LAYOUT =~ s/$TAG_START\s*([\w-]+)\s*:\s*(IF|UNLESS)\s*(\S+)\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*\2\s*$TAG_END/
        if ($2 eq "UNLESS") {
          print "- - - - - Parsing the unless \$VARIABLE{'$3'} statement... ";
          if ( (! exists $VARIABLE{$3}) || ($VARIABLE{$3} eq "0") || ($VARIABLE{$3} eq "") ) {
            print "True.\n";
            $4;
          } else {
            print "False.\n";
            ""
          }
        } else {
          print "- - - - - Parsing the if \$VARIABLE{'$3'} statement... ";
          if ( (exists $VARIABLE{$3}) && ( ($VARIABLE{$3} eq "1") || (($VARIABLE{$3} ne "0") && ($VARIABLE{$3} ne "")) ) ) {
            print "True.\n";
            $4;
          } else {
            print "False.\n";
            ""
          }
        }
      /isge
    ) {

    }
    print "- - - - Done.\n";

    print "- - - - Parsing the variables...\n";
    $LAYOUT =~ s/$TAG_START\s*([\w\.\-]+)\s*$TAG_END/
      print "- - - - - Parsing the \$VARIABLE{'$1'}... ";
      if ( (exists $VARIABLE{$1}) && ($VARIABLE{$1} ne "")) {
        print "Done.\n";
        $VARIABLE{$1};
      } else {
        print "Fail.\n";
        ""
      }
    /gsie;
    print "- - - - Done.\n";

  print "- - - Done.\n";

  print "- - - Compiling it to '$in{'TO'}'...\n";
  $TAG_START = quotemeta("<~");
  $TAG_END   = quotemeta("~>");

    print "- - - - Parsing dynamtic tags... ";
    $LAYOUT =~ s/\n/\\n\";\n\$HTML \.= \"/g;
    $LAYOUT =~ s/$TAG_START\s*([\w\.\-]+)\s*$TAG_END/\"\.(exists \$FORM->{\'$1\'} ? \$FORM->{\'$1\'} : "")\.\"/ig;

    while (
      $LAYOUT =~ s/$TAG_START\s*([\w-]+)\s*:\s*UNLESS\s*(\S+)\s+==\s+\\\"(\S+)\\\"\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*UNLESS\s*$TAG_END/\"\;\n\nif ( (\$FORM->{\'$2\'} ne \"$3\") {\n\n\$HTML \.= \"$4\";\n\n}\n\n\$HTML \.= \"/isg
    ) {

    }

    while (
      $LAYOUT =~ s/$TAG_START\s*([\w-]+)\s*:\s*IF\s*(\S+)\s+==\s+\\\"(\S+)\\\"\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*IF\s*$TAG_END/\"\;\n\nif ( (\$FORM->{\'$2\'} eq \"$3\") {\n\n\$HTML \.= \"$4\";\n\n}\n\n\$HTML \.= \"/isg
    ) {

    }

    while (
      $LAYOUT =~ s/$TAG_START\s*([\w-]+)\s*:\s*UNLESS\s*(\S+)\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*UNLESS\s*$TAG_END/\"\;\n\nif ( (! exists \$FORM->{\'$2\'}) || (\$FORM->{\'$2\'} eq \"0\") || (\$FORM->{\'$2\'} eq \"\") ) {\n\n\$HTML \.= \"$3\";\n\n}\n\n\$HTML \.= \"/isg
    ) {

    }

    while (
      $LAYOUT =~ s/$TAG_START\s*([\w-]+)\s*:\s*IF\s*(\S+)\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*IF\s*$TAG_END/\"\;\n\nif ( (exists \$FORM->{\'$2\'}) && ( (\$FORM->{\'$2\'} eq \"1\") || ((\$FORM->{\'$2\'} ne \"0\") && (\$FORM->{\'$2\'} ne \"\")) ) ) {\n\n\$HTML \.= \"$3\";\n\n}\n\n\$HTML \.= \"/isg
    ) {

    }

    $LAYOUT = "\$HTML \.= \"".$LAYOUT."\";\n";

    $LAYOUT =~ s/\$HTML \.= \"\\n\";\n//g;
    $LAYOUT =~ s/([^\\])\"\"\./$1/g;
    $LAYOUT =~ s/\.\"\"//g;
    print "Done.\n";

    print "- - - - Saving to '$in{'TO'}/UB_$LAYOUT_FILENAME.src'... ";
    chmod (0777, $in{'TO'});
    open (TEXT, ">".$in{'TO'}."/UB_".$LAYOUT_FILENAME.".src") || (print "Error: $!\n" and return);
      chmod (0766, $in{'TO'}."/UB_".$LAYOUT_FILENAME.".src");
      print TEXT "package ", $LAYOUT_FILENAME, ";\n\n";
      print TEXT "sub Parse {\n";
      print TEXT "my (\$FORM) = \@_;\n";
      print TEXT "my (\$HTML);\n\n";

      print TEXT $LAYOUT;

      print TEXT "\nreturn \$HTML;\n";
      print TEXT "}\n\n";

      print TEXT "\%", $LAYOUT_FILENAME, "::DYNAMIC = (\n";
      foreach (keys %LANGUAGE::DYNAMIC) {
        $LANGUAGE::DYNAMIC{$_} =~ s/\\/\\\\/g;
        $LANGUAGE::DYNAMIC{$_} =~ s/\~/\\\~/g;
        $LANGUAGE::DYNAMIC{$_} =~ s/\"/\\\"/g;
        #$LANGUAGE::DYNAMIC{$_} =~ s/\$/\\\$/g;
        $LANGUAGE::DYNAMIC{$_} =~ s/\@/\\\@/g;
        $LANGUAGE::DYNAMIC{$_} =~ s/\%/\\\%/g;
        print TEXT "  \"", $_, "\" => qq~", $LANGUAGE::DYNAMIC{$_}, "~,\n";
      }
      print TEXT ");\n\n";

      print TEXT "1;";
    close (TEXT);
    print "Done.\n";

  print "- - - Done.\n";

  return 1;
}

###############################################################################
1; # end of Compile.lib.pl file
###############################################################################