###############################################################################
# Database.lib.pl                                                             #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################

package DB;

BEGIN { @AnyDBM_File::ISA = qw(DB_File GDBM_File NDBM_File SDBM_File) }

use Fcntl;
use AnyDBM_File;

use strict;

###############################################################################
# Variables

$DB::Delimeter   = "|^|";
$DB::unDelimeter = "|&|";

###############################################################################
# CreateTable function
# - function of creating new table into database
sub CreateTable {
  my %args = (
    "TABLE"       => "",
    "VALUE"       => {},
    "PRIMARY_KEY" => "",
    @_
  );

  my $Table = $args{'TABLE'};

  unless (mkdir ("$SYSTEM::DB_PATH/$Table", $SYSTEM::PERMISSION)) {
    &UB::CGIError("Can't create a new table into database. $!", "$SYSTEM::DB_PATH/$Table");
  }

  chmod (0777, "$SYSTEM::DB_PATH/$Table");

  &Standard::lock("file.lock");

  &Standard::FileOpen(*TABLE, "w", "$SYSTEM::DB_PATH/".$Table.".cfg");
    print TABLE <<TEXT;
package DB;

\$DB::Path{'$Table'} = "\$SYSTEM::DB_PATH/$Table";

%{ \$DB::Table{'$Table'} } = (
TEXT
    my @Keys = sort { $args{'VALUE'}->{$a}[0] <=> $args{'VALUE'}->{$b}[0] } keys %{ $args{'VALUE'} };
    for (my $i = 0; $i <= $#Keys; $i++) {
      my $Key = $Keys[$i];
      my $Space;

      $Space = " " x ( 20 - length ($Key) );
      print TABLE "  ".$Key.$Space."=> [";

      $Space = " " x (2 - length ( $args{'VALUE'}->{$Key}[0] ) );
      print TABLE $Space.$args{'VALUE'}->{$Key}[0].", ";

      $Space = " " x (10 - length ( $args{'VALUE'}->{$Key}[1] ) );
      print TABLE $Space."\"".$args{'VALUE'}->{$Key}[1]."\", ";

      $Space = " " x (5 - length ( $args{'VALUE'}->{$Key}[2] ) );
      print TABLE $Space.$args{'VALUE'}->{$Key}[2].", ";

      $Space = "";
      print TABLE $Space.$args{'VALUE'}->{$Key}[3].", ";

      $Space = " " x (10 - length ( $args{'VALUE'}->{$Key}[4] ) );
      print TABLE $Space."\"".$args{'VALUE'}->{$Key}[4]."\"]";

      print TABLE "," unless $i == $#Keys;
      print TABLE "\n";
    }
    print TABLE <<TEXT;
);

\$DB::PrimaryKey{'$Table'} = "$args{'PRIMARY_KEY'}";

\$DB::Pointer{'$Table'}   = "";

foreach my \$Key ( sort { \$DB::Table{'$Table'}->{\$a}[0] <=> \$DB::Table{'$Table'}->{\$b}[0] } keys \%{ \$DB::Table{'$Table'} } ) {
  push ( \@{ \$DB::Columns{'$Table'} }, \$Key );
  if (\$Key eq \$DB::PrimaryKey{'$Table'}) {
    \$DB::Primary{'$Table'} = \$DB::Table{'$Table'}->{\$Key}[0];
  }
}

1;
TEXT
  close (TABLE);

  tie (my %DB, $AnyDBM_File::ISA[0], $SYSTEM::DB_PATH."/".$Table."/$Table.db", O_RDWR|O_CREAT, 0777) || &UB::CGIError("Can't open file for writing. $!"  , $SYSTEM::DB_PATH."/".$Table."/$Table.db");
  untie (%DB);

  &Standard::FileOpen(*TABLE, "w", $SYSTEM::DB_PATH."/".$Table."/".$Table.".counter");
    print TABLE "0";
  close (TABLE);

  &Standard::lock("file.lock");

  return 1;
}

###############################################################################
# Insert function
# - function of insert record into database table
sub Insert {
  my %args = (
    "TABLE"  => "",
    "SKIP "  =>  0,
    "VALUES" => {},
    @_
  );

  my $Table = $args{'TABLE'}; # set current table

  &Standard::Require ("$Table.cfg", "$SYSTEM::DB_PATH/$Table.cfg");
  &Standard::lock    ("$Table.lock");

  my $DBTable     = $DB::Table{ $Table };
  my $PrimaryKey  = $DB::PrimaryKey{ $Table };

  &Standard::FileOpen(*DB, "r", $DB::Path{ $Table }."/$Table.counter");
    my $Counter   = <DB>;
  close (DB);
  chomp ( $Counter );

  $Counter++;

  # checking the primary key of the table is it auto increase number
  if ( ($DBTable->{ $PrimaryKey }[1] eq "autonumber") && ($args{'SKIP'} == 0) ) {
     $args{'VALUES'}->{ $PrimaryKey } = $Counter;
  }

  # validate all data field is it currently
  return unless ( &myValidate( $Table, $args{'VALUES'} ) );

  # append data into data file
  tie (my %DB, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || &UB::CGIError("Can't open file for writing. $!"  , $DB::Path{ $Table }."/$Table.db");
    $DB{ $args{'VALUES'}->{ $PrimaryKey } } = &myEncodeData( $Table, $args{'VALUES'} );
  untie (%DB);

  chmod (0766, $DB::Path{ $Table }."/$Table.counter");

  # write data into counter file
  &Standard::FileOpen(*DB, "w", $DB::Path{ $Table }."/$Table.counter");
    print DB $Counter;
  close (DB);

  &Standard::unLock("$Table.lock");

  return $args{'VALUES'}->{ $PrimaryKey };
}

###############################################################################
# Update function
# - function of updating record from database table
sub Update {
  my %args = (
    "TABLE"  => "",
    "VALUES" => {},
    "KEY"    => "",
    "RESET"  =>  0,
    @_
  );

  if ($args{'KEY'} eq "") {
    $DB::Error = "No search key in Table '$args{'TABLE'}'.";
    return;
  }

  my $Table = $args{'TABLE'}; # set current table

  &Standard::Require ("$Table.cfg", "$SYSTEM::DB_PATH/$Table.cfg");
  &Standard::lock    ("$Table.lock");

  my $DBTable     = $DB::Table{ $Table };
  my $PrimaryKey  = $DB::PrimaryKey{ $Table };
  my $Hash        = {};

  my %DB = ();
  tie (my %DBM, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || &UB::CGIError("Can't open file for reading. $!"  , $DB::Path{ $Table }."/$Table.db");
    foreach my $KEY (keys %DBM) {
      $DB{$KEY} = $DBM{$KEY};
    }
  untie (%DBM);

  unless (exists ( $DB{ $args{'KEY'} } )) {
    $DB::Error = "Primary key not found in Table '$Table'.";
    return;
  }

  if ($args{'RESET'} == 0) {
    $Hash = &myDecodeData( $Table, $DB{ $args{'KEY'} } );

    foreach my $Key ( keys %{ $args{'VALUES'} } ) {
      next if ($Key eq $PrimaryKey);
      next if (($DB::Table{ $Table }->{$Key}[3] == 1) && ($args{'VALUES'}->{$Key} eq ""));

      if ($args{'VALUES'}->{$Key} =~ /(.*)\$\{(.+)\}(.*)/) {
        $args{'VALUES'}->{$Key} =~ s/\$\{([\.\w]+)\}/\$Hash->{\'$1\'}/ig;
        my $eval = qq~
          sub EvalData {
            my (\$Hash) = \@_;
            return ($args{'VALUES'}->{$Key});
          }
        ~;
        eval "$eval";
        $args{'VALUES'}->{$Key} = &EvalData($Hash);
      }
      $Hash->{$Key} = $args{'VALUES'}->{$Key};
    }
  } else {
    $Hash = $args{'VALUES'};
  }

  # validate all data field is it currently
  return unless ( &myValidate( $Table, $Hash, 1 ) );

  $DB{ $args{'KEY'} } = &myEncodeData( $Table, $Hash );

  tie (%DBM, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || &UB::CGIError("Can't open file for writing. $!"  , $DB::Path{ $Table }."/$Table.db");
    %DBM = ();
    foreach my $KEY (keys %DB) {
      $DBM{$KEY} = $DB{$KEY};
    }
  untie (%DB);

  &Standard::unLock("$Table.lock");

  return $Hash;
}

###############################################################################
# Delete function
# - function of deleting record from database table
sub Delete {
  my %args = (
    "TABLE" => "",
    "KEYS"  => [],
    @_
  );

  if ($#{ $args{'KEYS'} } < 0) {
    $DB::Error = "No search key.";
    return;
  }

  my $Table = $args{'TABLE'}; # set current table

  &Standard::Require ("$Table.cfg", "$SYSTEM::DB_PATH/$Table.cfg");
  &Standard::lock    ("$Table.lock");

  my $DBTable     = $DB::Table{ $Table };
  my $PrimaryKey  = $DB::PrimaryKey{ $Table };

  my %DB = ();
  tie (my %DBM, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || &UB::CGIError("Can't open file for reading. $!"  , $DB::Path{ $Table }."/$Table.db");
    foreach my $KEY (keys %DBM) {
      $DB{$KEY} = $DBM{$KEY};
    }
  untie (%DBM);

  for (my $i = 0; $i <= $#{ $args{'KEYS'} }; $i++) {
    unless (exists ( $DB{ $args{'KEYS'}->[$i] } )) {
      next;
    }

    delete ( $DB{ $args{'KEYS'}->[$i] } );
  }

  tie (%DBM, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || &UB::CGIError("Can't open file for writing. $!"  , $DB::Path{ $Table }."/$Table.db");
    %DBM = ();
    foreach my $KEY (keys %DB) {
      $DBM{$KEY} = $DB{$KEY};
    }
  untie (%DB);

  &Standard::unLock("$Table.lock");

  return 1;
}

###############################################################################
# Query function
# - function of searching records from database table
sub Query {
  my %args = (
    "TABLE"   => "",
    "COLUMNS" => [],
    "WHERE"   => {},
    "MATCH"   => "", # ALL/ANY
    "SORT"    => "",
    "BY"      => "",
    "TO"      => "",
    @_
  );

  my $Table = $args{'TABLE'}; # set current table

  &Standard::Require ("$Table.cfg", "$SYSTEM::DB_PATH/$Table.cfg");
  &Standard::lock    ("$Table.lock");

  my $DBTable     = $DB::Table      { $Table };
  my $PrimaryKey  = $DB::PrimaryKey { $Table };
  my $Columns     = $DB::Columns    { $Table };

  my (%Sort, @Return);

  my (%REString, %EQString, %NEString, %GTString, %LTString, %EQNumber, %NENumber, %GTNumber, %LTNumber);
  my (@Fields);
  if ( scalar @{ $args{'COLUMNS'} } > 0) {
    @Fields = @{ $args{'COLUMNS'} };
  } else {
    push ( @Fields, @{ $Columns } );
  }

  $args{'SORT'} = $PrimaryKey unless ($args{'SORT'});

  my $SearchFlag = 0;
  FOREACH: foreach my $Field (keys %{ $args{'WHERE'} } ) {
    FOR: for (my $i = 0; $i <= $#{ $args{'WHERE'}->{$Field} }; $i++) {
      next FOR if $args{'WHERE'}->{$Field}[$i] =~ /\*$/;
      my $Statement = $args{'WHERE'}->{$Field}[$i];

      if ($Statement =~ /^=~(.+)$/) {
        push (@{ $REString{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      }

      if ($Statement =~ /^>(\d+)$/) {
        push (@{ $GTNumber{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      } elsif ($Statement =~ /^>(.+)$/) {
        push (@{ $GTString{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      }

      if ($Statement =~ /^<(\d+)$/) {
        push (@{ $LTNumber{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      } elsif ($Statement =~ /^<(.+)$/) {
        push (@{ $LTString{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      }

      if ($Statement =~ /^!=(\d+)$/) {
        push (@{ $NENumber{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      } elsif ($Statement =~ /^!=(.+)$/) {
        push (@{ $NEString{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      }

      if ($Statement =~ /^=?(\d+)$/) {
        push (@{ $EQNumber{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      } elsif ($Statement =~ /^=?(.+)$/) {
        push (@{ $EQString{$Field} }, $1);
        $SearchFlag =  1;
        next FOR;
      }
    }
  }

  foreach my $Field (keys %REString) {
    for (my $i = 0; $i <= $#{ $REString{$Field} }; $i++) {
      my $tmp = "$REString{$Field}->[$i]";
      $REString{$Field}->[$i] = eval "sub { m/\Q$tmp\E/io }";
    }
  }

  my %DB = ();
  tie (my %DBM, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || (print "Error $!" and &UB::CGIError("Can't open file for reading. $!", $DB::Path{ $Table }."/$Table.db"));
    foreach my $KEY (keys %DBM) {
      $DB{$KEY} = $DBM{$KEY};
    }
  untie (%DBM);

  my $MatchAll  = 1 if ($args{'MATCH'} eq "ALL");
  my $FoundHits = 0;

  LINE: foreach my $Line ( values %DB ) {
    my $Matched = 0;
    my $Data    = &myDecodeData($Table, $Line);

    if ($SearchFlag == 0) {
      $Matched = 1;
      goto FOUND;
    }

    # searching fields equal to the regular expression
    foreach my $Field (keys %REString) {
      foreach my $Value ( @{ $REString{$Field} } ) {
        $_ = $Data->{ $Field };
        if (&{$Value}) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields equal to numbers
    foreach my $Field (keys %EQNumber) {
      foreach my $Value ( @{ $EQNumber{$Field} } ) {
        if ($Data->{ $Field } == $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields equal to strings
    foreach my $Field (keys %EQString) {
      foreach my $Value ( @{ $EQString{$Field} } ) {
        if ($Data->{ $Field } eq $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields not equal to numbers
    foreach my $Field (keys %NENumber) {
      foreach my $Value ( @{ $NENumber{$Field} } ) {
        if ($Data->{ $Field } != $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields not equal to strings
    foreach my $Field (keys %NEString) {
      foreach my $Value ( @{ $NEString{$Field} } ) {
        if ($Data->{ $Field } ne $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields greater then numbers
    foreach my $Field (keys %GTNumber) {
      foreach my $Value ( @{ $GTNumber{$Field} } ) {
        if ($Data->{ $Field } > $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields greater then strings
    foreach my $Field (keys %GTString) {
      foreach my $Value ( @{ $GTString{$Field} } ) {
        if ($Data->{ $Field } gt $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields less then numbers
    foreach my $Field (keys %LTNumber) {
      foreach my $Value ( @{ $LTNumber{$Field} } ) {
        if ($Data->{ $Field } < $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    # searching fields greater then strings
    foreach my $Field (keys %LTString) {
      foreach my $Value ( @{ $LTString{$Field} } ) {
        if ($Data->{ $Field } lt $Value) {
          $Matched = 1;
          goto FOUND unless $MatchAll;
        } else {
          next LINE if $MatchAll;
        }
      }
    }

    FOUND: # check is it matched
    if ($Matched == 1) {
      my %tmp;
      foreach (@Fields) {
        $tmp{$_} = $Data->{$_};
      }
      push (@Return, \%tmp);
      $Sort{$#Return} = $tmp{ $args{'SORT'} }; # if ($args{'SORT'} ne $PrimaryKey);
      $FoundHits++;
    }
  }

  return if ($FoundHits < 1);

  # sort the list of return
  my @tmp;
  if ($DBTable->{$args{'SORT'}}[1] eq "string") {
    if ($args{'BY'} eq "Z-A") {
      foreach my $i (sort { $Sort{$b} cmp $Sort{$a} } keys %Sort) {
        push (@tmp, $Return[$i]);
      }
    } else {
      foreach my $i (sort { $Sort{$a} cmp $Sort{$b} } keys %Sort) {
        push (@tmp, $Return[$i]);
      }
    }
  } else {
    if ($args{'BY'} eq "Z-A") {
      foreach my $i (sort { $Sort{$b} <=> $Sort{$a} } keys %Sort) {
        push (@tmp, $Return[$i]);
      }
    } else {
      foreach my $i (sort { $Sort{$a} <=> $Sort{$b} } keys %Sort) {
        push (@tmp, $Return[$i]);
      }
    }
  }

  @Return = @tmp;
  @tmp = ();

  if ($args{'TO'}) {
    my ($From, $To) = split (/-/, $args{'TO'});

    $From = 0        if ((!$From) || ($From < 0       ));
    $To   = $#Return if ((!$To  ) || ($To   > $#Return));

    @Return = @Return[$From..$To];
  }

  my %Index;
  for (my $i = 0; $i <= $#Return; $i++) {
    $Index{ $Return[$i]{ $PrimaryKey } } = $i;
  }

  &Standard::unLock("$Table.lock");

  return (\@Return, \%Index);
}

###############################################################################
# BinarySelect function
# - function of searching records from database table by using binary search
sub BinarySelect {
   my %args = (
    "TABLE"   => "",
    "COLUMNS" => [],
    "KEY"     => "",
    @_
  );

  if ($args{'KEY'} eq "") {
    $DB::Error = "No search key in Table '$args{'TABLE'}'.";
    return;
  }

  my $Table = $args{'TABLE'}; # set current table

  &Standard::Require ("$Table.cfg", "$SYSTEM::DB_PATH/$Table.cfg");
  &Standard::lock    ("$Table.lock");

  my $DBTable     = $DB::Table      { $Table };
  my $PrimaryKey  = $DB::PrimaryKey { $Table };
  my $Columns     = $DB::Columns    { $Table };

  my (@Fields);
  if ( scalar @{ $args{'COLUMNS'} } > 0) {
    @Fields = @{ $args{'COLUMNS'} };
  } else {
    push ( @Fields, @{ $Columns } );
  }

  my %DB = ();
  tie (my %DBM, $AnyDBM_File::ISA[0], $DB::Path{ $Table }."/$Table.db", O_RDWR|O_CREAT, 0777) || (print "Error $!" and &UB::CGIError("Can't open file for reading. $!", $DB::Path{ $Table }."/$Table.db"));
    foreach my $KEY (keys %DBM) {
      $DB{$KEY} = $DBM{$KEY};
    }
  untie (%DBM);

  unless (exists ( $DB{ $args{'KEY'} } )) {
    $DB::Error = "Data not found in Table '$Table'.";
    return;
  }

  my $Data = &myDecodeData($Table, $DB{ $args{'KEY'} });

  my %Return;
  foreach (@Fields) {
    $Return{$_} = $Data->{$_};
  }

  &Standard::unLock("$Table.lock");

  return \%Return;
}

###############################################################################
# myEncodeData function (Private)
# - function of encoding the data from input, and parse it into a line
sub myEncodeData {
  my ($Table, $Value) = @_;

  my $Return;
  my $Columns = $DB::Columns { $Table };

  foreach my $Column ( @{ $Columns } ) {
    my $tmp =  $Value->{$Column};

    $tmp    =~ s/^\s+//g;
    $tmp    =~ s/\s+$//g;

    $tmp    =~ s/\r//g;
    $tmp    =~ s/\n/\\n/g;

    $tmp    =~ s/\Q$DB::Delimeter\E/$DB::unDelimeter/g;

    $Return .= $tmp.$DB::Delimeter;
  }

  $Return =~ s/\Q$DB::Delimeter\E$//g;

  return $Return;
}

###############################################################################
# myDecodeData function (Private)
# - function of decoding the data from line, and parse it into a array
sub myDecodeData {
  my ($Table, $Data) = @_;
  my (@tmp, %Return);

  my $Columns = $DB::Columns { $Table };

  chomp ($Data);
  @tmp = split (/\Q$DB::Delimeter\E/, $Data, scalar ( $Columns ) );

  for (my $i = 0; $i <= $#{ $Columns }; $i++){
    $tmp[$i]  =~ s/\Q$DB::unDelimeter\E/$DB::Delimeter/g;
    $tmp[$i]  =~ s/\\n/\n/g;
    $Return{ $Columns->[$i] } = $tmp[$i];
  }

  return \%Return;
}

###############################################################################
# myValidate function (Private)
sub myValidate {
  my ($Table, $Value, $Update) = @_;

  $Update = 0 unless $Update;

  my $DBTable = $DB::Table { $Table };

  # validate all data field is it currently
  foreach my $Column ( @{ $DB::Columns{$Table} } ) {
    $Value->{ $Column } = "" unless exists ( $Value->{ $Column } );
    $Value->{ $Column } =~ s/\r//g;

    if ($DBTable->{$Column}[4] ne "") { # parsing the default values
      if ( $Value->{ $Column } eq "" ) {
        $Value->{ $Column } = $DBTable->{$Column}[4];
      }
    }

    if (($DBTable->{$Column}[3] == 1) && ($Update != 1)) { # checking required fields
      if ( $Value->{ $Column } eq "" ) {
        $DB::Error = "Missing value for '$Column' field in table '$Table'.";
        return;
      }
    }

    if ($DBTable->{$Column}[2] > 0) { # checking length of fields
      if (
        ( $Value->{ $Column } eq "" ) &&
        ( length ( $Value->{ $Column } ) > $DBTable->{ $Column }[2] ) &&
        ( $DBTable->{$Column}[2] != -1 )
      ) {
        $DB::Error = "The length of '$Column' field is too long in table '$Table'.";
        return;
      }
    }

    if ($DBTable->{$Column}[1] eq "number") { # checking format of fields
      if ( ($Value->{ $Column } !~ /^[\-\d\.]+$/) && ( $Value->{ $Column } ne "" ) ) {
        $DB::Error = "The format of '$Column' field is wrong in table '$Table'.";
        return;
      }
    }
  }

  return 1;
}

###############################################################################
1; # end of Database.lib.pl file
###############################################################################