###############################################################################
# Move.lib.pl                                                                 #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################
package Move;

use strict;
use File::Copy;

&Standard::Require("Thread.lib.pl", "$UB::LIBRARIES_PATH/Thread.lib.pl");

sub Topic {
  my %in = (
    FROM_BOARD => "",
    FROM_TOPIC => "",
    TO_BOARD   => "",
    DETAIL     =>  0,
    @_
  );

  my $T;
  my $LAST_POST_SECOND = 0;

  print "<!--\n" if $in{'DETAIL'} != 1;

  if ( ($in{'FROM_BOARD'} eq "") || ($in{'FROM_TOPIC'} eq "") || ($in{'TO_BOARD'} eq "") ) {
    print "! Missing input value.\n";
    return -1;
  }
  print "- Begin moving topic id# $in{'FROM_TOPIC'} in board id# $in{'FROM_BOARD'} to board id# $in{'TO_BOARD'}.\n";

  $T = time;
  print "- Getting topic information... ";
  (
    my ($Topic) = &DB::BinarySelect (
      TABLE   => "B".$in{'FROM_BOARD'},
      KEY     => $in{'FROM_TOPIC'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  my @Replies = split (/,/, $Topic->{'REPLIES'});
  shift @Replies;

  my @NEW_REPLIES;

  $T = time;
  print "- Inserting topic information into board id# $in{'TO_BOARD'}... ";
  ( # insert topic information into board database
    my $NEW_TOPIC_ID = &DB::Insert (
      TABLE  => "B".$in{'TO_BOARD'},
      VALUES => {
        SUBJECT             => $Topic->{'SUBJECT'},
        USERNAME            => $Topic->{'USERNAME'},
        EMAIL               => $Topic->{'EMAIL'},
        NICKNAME            => $Topic->{'NICKNAME'},
        MEMBER_LEVEL        => $Topic->{'MEMBER_LEVEL'},
        MEMBER_GROUP        => $Topic->{'MEMBER_GROUP'},
        ROOT                => 0,
        REPLIES             => "",
        NUMBER_REPLIES      => 0,
        POST_SECOND         => $Topic->{'POST_SECOND'},
        POST_DATE           => $Topic->{'POST_DATE'},
        POST_TIME           => $Topic->{'POST_TIME'},
        LAST_POST_SECOND    => $Topic->{'LAST_POST_SECOND'},
        LAST_POST_DATE      => $Topic->{'LAST_POST_DATE'},
        LAST_POST_TIME      => $Topic->{'LAST_POST_TIME'},
        LAST_UPDATE_SECOND  => $Topic->{'LAST_UPDATE_SECOND'},
        LAST_UPDATE_DATE    => $Topic->{'LAST_UPDATE_DATE'},
        LAST_UPDATE_TIME    => $Topic->{'LAST_UPDATE_TIME'},
        TOTAL_MODIFIED      => $Topic->{'TOTAL_MODIFIED'},
        HOST                => $Topic->{'HOST'},
        IP                  => $Topic->{'IP'},
        REPLY_NOTIFY        => $Topic->{'REPLY_NOTIFY'},
        SIGNATURE           => $Topic->{'SIGNATURE'},
        SYMBOL              => $Topic->{'SYMBOL'},
        APROVED             => $Topic->{'APROVED'},
        READ_ONLY           => $Topic->{'READ_ONLY'},
        ATTACHMENT          => $Topic->{'ATTACHMENT'}
      }
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $LAST_POST_SECOND = $Topic->{'LAST_POST_SECOND'} if $Topic->{'LAST_POST_SECOND'} > $LAST_POST_SECOND;

  $T = time;
  print "- Moving the topic html source into board id# $in{'TO_BOARD'}... ";
  my $Counter = 0;
  while (move ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}."-".++$Counter.".html" , $SYSTEM::DB_PATH."/B".$in{'TO_BOARD'}."/".$NEW_TOPIC_ID."-".$Counter.".html")) {}
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Moving the topic print source into board id# $in{'TO_BOARD'}... ";
  move ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}.".print" , $SYSTEM::DB_PATH."/B".$in{'TO_BOARD'}."/".$NEW_TOPIC_ID.".print");
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Moving the topic body source into board id# $in{'TO_BOARD'}... ";
  move ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}.".src" , $SYSTEM::DB_PATH."/B".$in{'TO_BOARD'}."/".$NEW_TOPIC_ID.".src") || print "Error: $! ";
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Moving the topic body parsed source into board id# $in{'TO_BOARD'}... ";
  move ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}.".post", $SYSTEM::DB_PATH."/B".$in{'TO_BOARD'}."/".$NEW_TOPIC_ID.".post") || print "Error: $! ";
  print "Done. (", (time - $T), "s)\n";

  if ($Topic->{'ATTACTMENT'}) {
    $T = time;
    print "- Moving the attachment into board id# $in{'TO_BOARD'}... ";
    if (-e $SYSTEM::UPLOAD_PATH."/Post-".$in{'FROM_BOARD'}."-".$in{'FROM_TOPIC'}."-".$Topic->{'ATTACTMENT'}) {
      move ($SYSTEM::UPLOAD_PATH."/Post-".$in{'FROM_BOARD'}."-".$in{'FROM_TOPIC'}."-".$Topic->{'ATTACTMENT'}, $SYSTEM::UPLOAD_PATH."/Post-".$in{'TO_BOARD'}."-".$NEW_TOPIC_ID."-".$Topic->{'ATTACTMENT'}) || print "Error: $! ";
    } else {
      print "Error: cannot find the attachment from board id# $in{'FROM_BOARD'}. Path: ".$SYSTEM::UPLOAD_PATH."/Post-".$in{'FROM_BOARD'}."-".$in{'FROM_TOPIC'}."-".$Topic->{'ATTACTMENT'}." ";
    }
    print "Done. (", (time - $T), "s)\n";
  }

  $T = time;
  print "- Moving all the replies into board id# $in{'TO_BOARD'}...\n";
  foreach my $REPLY_ID (@Replies) {
    my $T = time;
    print "- - Getting reply id# $REPLY_ID information...";
    (
      my ($Reply) = &DB::BinarySelect (
        TABLE   => "B".$in{'FROM_BOARD'},
        KEY     => $REPLY_ID
      )
    ) || (print "Error: $DB::Error\n" and return -5);
    print "Done. (", (time - $T), "s)\n";

    $T = time;
    print "- - Inserting reply id# $REPLY_ID information into board id# $in{'TO_BOARD'}... ";
    ( # insert post information into board database
      my $NEW_REPLY_ID = &DB::Insert (
        TABLE  => "B".$in{'TO_BOARD'},
        VALUES => {
          SUBJECT             => $Reply->{'SUBJECT'},
          USERNAME            => $Reply->{'USERNAME'},
          EMAIL               => $Reply->{'EMAIL'},
          NICKNAME            => $Reply->{'NICKNAME'},
          MEMBER_LEVEL        => $Reply->{'MEMBER_LEVEL'},
          MEMBER_GROUP        => $Reply->{'MEMBER_GROUP'},
          ROOT                => $NEW_TOPIC_ID,
          REPLIES             => "",
          NUMBER_REPLIES      => 0,
          POST_SECOND         => $Reply->{'POST_SECOND'},
          POST_DATE           => $Reply->{'POST_DATE'},
          POST_TIME           => $Reply->{'POST_TIME'},
          LAST_POST_SECOND    => $Reply->{'LAST_POST_SECOND'},
          LAST_POST_DATE      => $Reply->{'LAST_POST_DATE'},
          LAST_POST_TIME      => $Reply->{'LAST_POST_TIME'},
          LAST_UPDATE_SECOND  => $Reply->{'LAST_UPDATE_SECOND'},
          LAST_UPDATE_DATE    => $Reply->{'LAST_UPDATE_DATE'},
          LAST_UPDATE_TIME    => $Reply->{'LAST_UPDATE_TIME'},
          TOTAL_MODIFIED      => $Reply->{'TOTAL_MODIFIED'},
          HOST                => $Reply->{'HOST'},
          IP                  => $Reply->{'IP'},
          REPLY_NOTIFY        => $Reply->{'REPLY_NOTIFY'},
          SIGNATURE           => $Reply->{'SIGNATURE'},
          SYMBOL              => $Reply->{'SYMBOL'},
          APROVED             => $Reply->{'APROVED'},
          READ_ONLY           => $Reply->{'READ_ONLY'},
          ATTACHMENT          => $Reply->{'ATTACHMENT'}
        }
      )
    ) || (print "Error: $DB::Error\n" and return -5);
    print "Done. (", (time - $T), "s)\n";

    $LAST_POST_SECOND = $Reply->{'LAST_POST_SECOND'} if $Reply->{'LAST_POST_SECOND'} > $LAST_POST_SECOND;

    $T = time;
    print "- - Moving the reply id# $REPLY_ID body source into board id# $in{'TO_BOARD'}... ";
    move ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$REPLY_ID.".src" , $SYSTEM::DB_PATH."/B".$in{'TO_BOARD'}."/".$NEW_REPLY_ID.".src") || print "Error: $! ";
    print "Done. (", (time - $T), "s)\n";

    $T = time;
    print "- - Moving the reply id# $REPLY_ID body parsed source into board id# $in{'TO_BOARD'}... ";
    move ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$REPLY_ID.".post", $SYSTEM::DB_PATH."/B".$in{'TO_BOARD'}."/".$NEW_REPLY_ID.".post") || print "Error: $! ";
    print "Done. (", (time - $T), "s)\n";

    if ($Topic->{'ATTACTMENT'}) {
      $T = time;
      print "- - Moving the attachment into board id# $in{'TO_BOARD'}... ";
      if (-e $SYSTEM::UPLOAD_PATH."/Post-".$in{'FROM_BOARD'}."-".$REPLY_ID."-".$Reply->{'ATTACTMENT'}) {
        move ($SYSTEM::UPLOAD_PATH."/Post-".$in{'FROM_BOARD'}."-".$REPLY_ID."-".$Reply->{'ATTACTMENT'}, $SYSTEM::UPLOAD_PATH."/Post-".$in{'TO_BOARD'}."-".$NEW_REPLY_ID."-".$Reply->{'ATTACTMENT'}) || print "Error: $! ";
      } else {
        print "Error: cannot find the attachment from board id# $in{'FROM_BOARD'}. Path: ".$SYSTEM::UPLOAD_PATH."/Post-".$in{'FROM_BOARD'}."-".$REPLY_ID."-".$Reply->{'ATTACTMENT'}." ";
      }
      print "Done. (", (time - $T), "s)\n";
    }

    push (@NEW_REPLIES, $NEW_REPLY_ID);
  }
  print "- Done. (", (time - $T), "s)\n";

  if (scalar (@NEW_REPLIES) > 0) {
    $T = time;
    print "- Updating topic information from board id# $in{'TO_BOARD'}... ";
    (
      &DB::Update (
        TABLE   => "B".$in{'TO_BOARD'},
        VALUES  => {
          REPLIES        => ",".join (",", @NEW_REPLIES),
          NUMBER_REPLIES => scalar (@NEW_REPLIES)
        },
        KEY     => $NEW_TOPIC_ID
      )
    ) || (print "Error: $DB::Error\n" and return -5);
    print "Done. (", (time - $T), "s)\n";
  }

  $T = time;
  print "- Creating topic static html source to board id# $in{'TO_BOARD'}... ";
  &Thread::Build (
    BOARD     => $in{'TO_BOARD'},
    THREAD    => $NEW_TOPIC_ID,
  );
  print "Done. (", (time - $T), "s)\n";

  my $LAST_POST_DATE = &Standard::ConvertEpochToDate($LAST_POST_SECOND);
  my $LAST_POST_TIME = &Standard::ConvertEpochToTime($LAST_POST_SECOND);
  my $Posts;
  my $Board;

  $T = time;
  print "- Getting board id# $in{'TO_BOARD'} information... ";
  (
    ($Board) = &DB::BinarySelect (
      TABLE   => "Boards",
      COLUMNS => ["ID", "CATEGORY"],
      KEY     => $in{'TO_BOARD'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $Posts = scalar (@NEW_REPLIES) + 1;

  $T = time;
  print "- Updating category id# $Board->{'CATEGORY'} information... ";
  (
    &DB::Update (
      TABLE   => "Categories",
      VALUES  => {
        TOTAL_POSTS         => "\${TOTAL_POSTS} + $Posts" ,
        TOTAL_TOPICS        => "\${TOTAL_TOPICS} + 1"     ,

        LAST_UPDATE_SECOND  => "(\${LAST_UPDATE_SECOND} < $LAST_POST_SECOND) ? \"$LAST_POST_SECOND\" : \${LAST_UPDATE_SECOND}",
        LAST_UPDATE_DATE    => "(\${LAST_UPDATE_SECOND} < $LAST_POST_SECOND) ?   \"$LAST_POST_DATE\" :   \${LAST_UPDATE_DATE}",
        LAST_UPDATE_TIME    => "(\${LAST_UPDATE_SECOND} < $LAST_POST_SECOND) ?   \"$LAST_POST_TIME\" :   \${LAST_UPDATE_TIME}"
      },
      KEY     => $Board->{'CATEGORY'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Updating board id# $Board->{'ID'} information... ";
  (
    &DB::Update (
      TABLE   => "Boards",
      VALUES  => {
        TOTAL_POSTS         => "\${TOTAL_POSTS} + $Posts" ,
        TOTAL_TOPICS        => "\${TOTAL_TOPICS} + 1"     ,

        LAST_POST_SECOND    => "(\${LAST_POST_SECOND} < $LAST_POST_SECOND) ? \"$LAST_POST_SECOND\" : \${LAST_POST_SECOND}",
        LAST_POST_DATE      => "(\${LAST_POST_SECOND} < $LAST_POST_SECOND) ?   \"$LAST_POST_DATE\" :   \${LAST_POST_DATE}",
        LAST_POST_TIME      => "(\${LAST_POST_SECOND} < $LAST_POST_SECOND) ?   \"$LAST_POST_TIME\" :   \${LAST_POST_TIME}",

        LAST_UPDATE_SECOND  => "(\${LAST_UPDATE_SECOND} < $LAST_POST_SECOND) ? \"$LAST_POST_SECOND\" : \${LAST_UPDATE_SECOND}",
        LAST_UPDATE_DATE    => "(\${LAST_UPDATE_SECOND} < $LAST_POST_SECOND) ?   \"$LAST_POST_DATE\" :   \${LAST_UPDATE_DATE}",
        LAST_UPDATE_TIME    => "(\${LAST_UPDATE_SECOND} < $LAST_POST_SECOND) ?   \"$LAST_POST_TIME\" :   \${LAST_UPDATE_TIME}"
      },
      KEY     => $Board->{'ID'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Removing topic id# $in{'FROM_TOPIC'}, and replies id# (".join (", ", @Replies).") information... ";
  (
    &DB::Delete (
      TABLE  => "B".$in{'FROM_BOARD'},
      KEYS   => [$in{'FROM_TOPIC'}, @Replies]
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Removing topic id# $in{'FROM_TOPIC'} static html...\n";
  my $P = 0;
  while ( unlink ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}."-".++$P.".topic") ) {
    print "- - Removed '".$SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}."-".$P.".topic'\n";
  }
  $P = 0;
  while ( unlink ($SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}."-".++$P.".print") ) {
    print "- - Removed '".$SYSTEM::DB_PATH."/B".$in{'FROM_BOARD'}."/".$in{'FROM_TOPIC'}."-".$P.".print'\n";
  }
  print "- Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Getting board id# $in{'FROM_BOARD'} information... ";
  (
    ($Board) = &DB::BinarySelect (
      TABLE   => "Boards",
      COLUMNS => ["ID", "CATEGORY"],
      KEY     => $in{'FROM_BOARD'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $Posts = scalar (@NEW_REPLIES) + 1;

  $T = time;
  print "- Updating category id# $Board->{'CATEGORY'} information... ";
  (
    &DB::Update (
      TABLE   => "Categories",
      VALUES  => {
        TOTAL_POSTS         => "\${TOTAL_POSTS} - $Posts" ,
        TOTAL_TOPICS        => "\${TOTAL_TOPICS} - 1"
      },
      KEY     => $Board->{'CATEGORY'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Updating board id# $Board->{'ID'} information... ";
  (
    &DB::Update (
      TABLE   => "Boards",
      VALUES  => {
        TOTAL_POSTS         => "\${TOTAL_POSTS} - $Posts" ,
        TOTAL_TOPICS        => "\${TOTAL_TOPICS} - 1"
      },
      KEY     => $Board->{'ID'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  print "-->\n"  if $in{'DETAIL'} != 1;

  return $NEW_TOPIC_ID;
}

###############################################################################
1; # end of Move.lib.pl file
###############################################################################