###############################################################################
# Rebuild.lib.pl                                                              #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################
package Rebuild;

use strict;
use Benchmark;

###############################################################################
# Database function
sub Database {
  my %in = (
    DETAIL     =>  1,
    @_
  );

  my %Stats = ();
  my ($T0, $T1);

  # Groups.cfg ################################################################
  $T0 = new Benchmark;
  print "- Reading the member groups database...\n";
  (
    my ($Groups, $GroupsIndex) = &DB::Query (
      TABLE   => "Groups"
    )
  ) || print "- Error: \"$DB::Error\".\n";

    print "- - Checking the defualt member group... ";
    unless ( exists ( $GroupsIndex->{ $GENERAL::DEFAULT_GROUP } ) ) {
      print "incorrect, fix: re-build the default member group data... ";

      push (
        @{ $Groups },
        {
          ID                => $GENERAL::DEFAULT_GROUP,
          TITLE             => "Default Member Group",
          DESCRIPTION       => "Default Member Group",
          MAX_FILE_SIZE     => "0",
          TOTAL_MEMBERS     => "0"
        }
      );
      $GroupsIndex->{ $GENERAL::DEFAULT_GROUP } = $#{ $Groups };

      if (
        &DB::Insert (
          "TABLE"  => "Groups",
          "SKIP "  => 1,
          "VALUES" => $Groups->[ $#{ $Groups } ]
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
        return;
      }
    } else {
      print "correct.\n";
    }

    for (my $g = 0; $g <= $#{ $Groups }; $g++) {
      $Stats{'GROUP'}->{ $Groups->[$g]{'ID'} } = 0;
    }

  $T1 = new Benchmark;
  print "- Done. Used".timestr( timediff($T1, $T0) )."\n\n";

  # Accounts.cfg ##############################################################
  $T0 = new Benchmark;
  print "- Reading the member accounts database...\n";
  (
    my ($Accounts, $AccountsIndex) = &DB::Query (
      TABLE   => "Accounts"
    )
  ) || print "- Error: \"$DB::Error\".\n";

  for (my $a = 0; $a <= $#{ $Accounts }; $a++) {
    my $Account = $Accounts->[$a];
    my $Fix = 0;

    print "- - ".($a + 1).")Checking member <".$Account->{'USERNAME'}."> account... ";

    if (($Account->{'STATUS'} ne "30") && ($Account->{'STATUS'} ne "50")) {
      print "status is incorrect, fix: replace to 50... ";
      $Account->{'STATUS'} = 30;
      $Fix++;
    }

    if (($Account->{'LEVEL'} ne "30") && ($Account->{'LEVEL'} ne "50") && ($Account->{'LEVEL'} ne "70") && ($Account->{'LEVEL'} ne "100")) {
      print "level is incorrect, fix: replace to 30... ";
      $Account->{'LEVEL'} = 30;
      $Fix++;
    }

    unless (exists ( $GroupsIndex->{ $Account->{'GROUP'} } ) ) {
      print "group is incorrect, fix: replace to $GENERAL::DEFAULT_GROUP... ";
      $Account->{'GROUP'} = $GENERAL::DEFAULT_GROUP;
      $Fix++;
    } else {
      if ($Account->{'GROUP_TEXT'} ne $Groups->[$GroupsIndex->{ $Account->{'GROUP'} }]{'TITLE'}) {
        print "group name is different, update: replace to ".$Groups->[$GroupsIndex->{ $Account->{'GROUP'} }]{'TITLE'}."... ";
        $Account->{'GROUP_TEXT'} = $Groups->[$GroupsIndex->{ $Account->{'GROUP'} }]{'TITLE'};
        $Fix++;
      }
    }

    if ($Account->{'LEVEL'} eq "30") {
      if ($Account->{'LEVEL_TEXT'} ne $GENERAL::MEMBER_LEVEL_30) {
        print "level name is out dated, update: replace to $GENERAL::MEMBER_LEVEL_30... ";
        $Account->{'LEVEL_TEXT'} = $GENERAL::MEMBER_LEVEL_30;
        $Fix++;
      }
    } elsif ($Account->{'LEVEL'} eq "50") {
      if ($Account->{'LEVEL_TEXT'} ne $GENERAL::MEMBER_LEVEL_50) {
        print "level name is out dated, update: replace to $GENERAL::MEMBER_LEVEL_50... ";
        $Account->{'LEVEL_TEXT'} = $GENERAL::MEMBER_LEVEL_50;
        $Fix++;
      }
    } elsif ($Account->{'LEVEL'} eq "70") {
      if ($Account->{'LEVEL_TEXT'} ne $GENERAL::MEMBER_LEVEL_70) {
        print "level name is out dated, update: replace to $GENERAL::MEMBER_LEVEL_70... ";
        $Account->{'LEVEL_TEXT'} = $GENERAL::MEMBER_LEVEL_70;
        $Fix++;
      }
    } elsif ($Account->{'LEVEL'} eq "100") {
      if ($Account->{'LEVEL_TEXT'} ne $GENERAL::MEMBER_LEVEL_100) {
        print "level name is out dated, update: replace to $GENERAL::MEMBER_LEVEL_100... ";
        $Account->{'LEVEL_TEXT'} = $GENERAL::MEMBER_LEVEL_100;
        $Fix++;
      }
    }

    if ($Account->{'TOTAL_POSTS'} < 0) {
      print "total posts is incorrect, fix: replace to 0... ";
      $Account->{'TOTAL_POSTS'} = 0;
      $Fix++;
    }

    my ($Date, $Time);

    if ($Account->{'REGISTERED_SECOND'}) {
      $Date = &Standard::ConvertEpochToDate( $Account->{'REGISTERED_SECOND'} );
      $Time = &Standard::ConvertEpochToTime( $Account->{'REGISTERED_SECOND'} );
      if (
        ( $Account->{'REGISTERED_DATE'} ne $Date ) ||
        ( $Account->{'REGISTERED_TIME'} ne $Time )
      ) {
        print "registered date/time is out dated, update: replace to $Date/$Time... ";
        $Account->{'REGISTERED_DATE'} = $Date;
        $Account->{'REGISTERED_TIME'} = $Time;
        $Fix++;
      }
    }

    if ($Account->{'LAST_POST_SECOND'}) {
      $Date = &Standard::ConvertEpochToDate( $Account->{'LAST_POST_SECOND'} );
      $Time = &Standard::ConvertEpochToTime( $Account->{'LAST_POST_SECOND'} );
      if (
        ( $Account->{'LAST_POST_DATE'} ne $Date ) ||
        ( $Account->{'LAST_POST_TIME'} ne $Time )
      ) {
        print "last post date/time is out dated, update: replace to $Date/$Time... ";
        $Account->{'LAST_POST_DATE'} = $Date;
        $Account->{'LAST_POST_TIME'} = $Time;
        $Fix++;
      }
    }

    if ($Account->{'LAST_LOGIN_SECOND'}) {
      $Date = &Standard::ConvertEpochToDate( $Account->{'LAST_LOGIN_SECOND'} );
      $Time = &Standard::ConvertEpochToTime( $Account->{'LAST_LOGIN_SECOND'} );
      if (
        ( $Account->{'LAST_LOGIN_DATE'} ne $Date ) ||
        ( $Account->{'LAST_LOGIN_TIME'} ne $Time )
      ) {
        print "last login date/time is out dated, update: replace to $Date/$Time... ";
        $Account->{'LAST_LOGIN_DATE'} = $Date;
        $Account->{'LAST_LOGIN_TIME'} = $Time;
        $Fix++;
      }
    }

    if ($Fix > 0) {
      print "updating... ";
      if (
        &DB::Update (
          TABLE   => "Accounts",
          VALUES  => $Account,
          KEY     => $Account->{'USERNAME'}
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
      }
    } else {
      print "correct.\n";
    }

    $Stats{'GROUP'}->{ $Account->{'GROUP'} }++;
  }

  $T1 = new Benchmark;
  print "- Done. Used".timestr( timediff($T1, $T0) )."\n\n";

  # Profiles.cfg ##############################################################
  $T0 = new Benchmark;
  print "- Reading the member profiles database...\n";
  (
    my ($Profiles, $ProfilesIndex) = &DB::Query (
      TABLE   => "Profiles"
    )
  ) || print "- Error: \"$DB::Error\".\n";

  for (my $a = 0; $a <= $#{ $Accounts }; $a++) {
    my $Account = $Accounts->[$a];

    print "- - ".($a + 1).")Checking member <".$Account->{'USERNAME'}."> profile... ";

    unless ( exists ( $ProfilesIndex->{ $Account->{'USERNAME'} } ) ) {
      print "not found, fix: re-create a new profile... ";
      push (
        @{ $Profiles },
        {
          USERNAME => $Account->{'USERNAME'}
        }
      );
      $ProfilesIndex->{ $Account->{'USERNAME'} } = $#{ $Profiles };

      if (
        &DB::Insert (
          "TABLE"  => "Profiles",
          "VALUES" => $Profiles->[ $#{ $Profiles } ]
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
        return;
      }
    } else {
      print "correct.\n";
    }
  }

  $T1 = new Benchmark;
  print "- Done. Used".timestr( timediff($T1, $T0) )."\n\n";

  # Categories.cfg ############################################################
  $T0 = new Benchmark;
  print "- Reading the categories database...\n";
  (
    my ($Categories, $CategoriesIndex) = &DB::Query (
      TABLE   => "Categories"
    )
  ) || print "- Error: \"$DB::Error\".\n";

  print "- - Checking the defualt category... ";
  unless ( exists ( $CategoriesIndex->{ '1' } ) ) {
    print "incorrect, fix: re-build the default category data... ";

    push (
      @{ $Categories },
      {
        ID                  => "1",
        TITLE               => "Default Category",
        DESCRIPTION         => "Default Category"
      }
    );
    $CategoriesIndex->{ '1' } = $#{ $Categories };

    if (
      &DB::Insert (
        "TABLE"  => "Categories",
        "SKIP "  => 1,
        "VALUES" => $Categories->[ $#{ $Categories } ]
      )
    ) {
      print "success.\n";
    } else {
      print "error: \"$DB::Error\".\n";
      return;
    }
  } else {
    print "correct.\n";
  }

  for (my $c = 0; $c <= $#{ $Categories }; $c++) {
    my $Category = $Categories->[$c];
    my $Fix = 0;

    print "- - ".($c + 1).")Checking category #id ".$Category->{'ID'}." <".$Category->{'TITLE'}.">... ";

    if ($Category->{'TOTAL_BOARDS'} < 0) {
      print "total boards is incorrect, fix: replace to 0... ";
      $Category->{'TOTAL_BOARDS'} = 0;
      $Fix++;
    }

    if ($Category->{'TOTAL_POSTS'} < 0) {
      print "total posts is incorrect, fix: replace to 0... ";
      $Category->{'TOTAL_POSTS'} = 0;
      $Fix++;
    }

    if ($Category->{'TOTAL_TOPICS'} < 0) {
      print "total topics is incorrect, fix: replace to 0... ";
      $Category->{'TOTAL_TOPICS'} = 0;
      $Fix++;
    }

    my ($Date, $Time);

    if ($Category->{'LAST_UPDATE_SECOND'}) {
      $Date = &Standard::ConvertEpochToDate( $Category->{'LAST_UPDATE_SECOND'} );
      $Time = &Standard::ConvertEpochToTime( $Category->{'LAST_UPDATE_SECOND'} );
      if (
        ( $Category->{'LAST_UPDATE_DATE'} ne $Date ) ||
        ( $Category->{'LAST_UPDATE_TIME'} ne $Time )
      ) {
        print "last updated date/time is out dated, update: replace to $Date/$Time... ";
        $Category->{'LAST_UPDATE_DATE'} = $Date;
        $Category->{'LAST_UPDATE_TIME'} = $Time;
        $Fix++;
      }
    }

    if ($Fix > 0) {
      print "updating... ";
      if (
        &DB::Update (
          TABLE   => "Categories",
          VALUES  => $Category,
          KEY     => $Category->{'ID'}
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
      }
    } else {
      print "correct.\n";
    }

    $Stats{'CATEGORY'}->{ $Category->{'ID'} }{'BOARDS'} = 0;
    $Stats{'CATEGORY'}->{ $Category->{'ID'} }{'TOPICS'} = 0;
    $Stats{'CATEGORY'}->{ $Category->{'ID'} }{'POSTS'}  = 0;
  }

  $T1 = new Benchmark;
  print "- Done. Used".timestr( timediff($T1, $T0) )."\n\n";

  # Boards.cfg ################################################################
  $T0 = new Benchmark;
  print "- Reading the boards database...\n";
  (
    my ($Boards, $BoardsIndex) = &DB::Query (
      TABLE   => "Boards"
    )
  ) || print "- Error: \"$DB::Error\".\n";

  for (my $b = 0; $b <= $#{ $Boards }; $b++) {
    my $Board = $Boards->[$b];
    my $Fix = 0;

    print "- - ".($b + 1).")Checking board #id ".$Board->{'ID'}." <".$Board->{'TITLE'}.">... ";

    unless ( exists ( $CategoriesIndex->{ $Board->{'CATEGORY'} } ) ) {
      print "category id# is not found, fix: replace to id# 1... ";
      $Board->{'CATEGORY'} = 1;
      $Fix++;
    }

    if ($Board->{'MAJOR_MODERATOR'}) {
      unless ( exists ( $AccountsIndex->{ $Board->{'MAJOR_MODERATOR'} } ) ) {
        print "major moderator is not found, fix: remove it... ";
        $Board->{'MAJOR_MODERATOR'}          = "";
        $Board->{'MAJOR_MODERATOR_NICKNAME'} = "";
        $Board->{'MAJOR_MODERATOR_EMAIL'}    = "";
        $Fix++;
      } else {
        my $Account = $Accounts->[ $AccountsIndex->{ $Board->{'MAJOR_MODERATOR'} } ];
        if ($Board->{'MAJOR_MODERATOR_NICKNAME'} ne $Account->{'NICKNAME'}) {
          print "major moderator nickname is out dated, update: replace to $Account->{'NICKNAME'}... ";
          $Board->{'MAJOR_MODERATOR_NICKNAME'} = $Account->{'NICKNAME'};
          $Fix++;
        }

        if ($Board->{'MAJOR_MODERATOR_EMAIL'} ne $Account->{'EMAIL'}) {
          print "major moderator email is out dated, update: replace to $Account->{'EMAIL'}... ";
          $Board->{'MAJOR_MODERATOR_EMAIL'} = $Account->{'EMAIL'};
          $Fix++;
        }
      }
    }

    my @MinorModerators = split (/,/, $Board->{'MINOR_MODERATORS'});
    my @tmp;
    for (my $m = 0; $m <= $#MinorModerators; $m++) {
      my $Moderator = $MinorModerators[$m];
      unless ( exists ( $AccountsIndex->{ $Moderator } ) ) {
        print "minor moderator '$Moderator' is not found, fix: remove it... ";
        $Fix++;
      } else {
        push (@tmp, $Moderator);
      }
    }
    @MinorModerators = @tmp;
    $Board->{'MINOR_MODERATORS'} = join (",", @MinorModerators);

    if ($Board->{'TOTAL_POSTS'} < 0) {
      print "total posts is incorrect, fix: replace to 0... ";
      $Board->{'TOTAL_POSTS'} = 0;
      $Fix++;
    }

    if ($Board->{'TOTAL_TOPICS'} < 0) {
      print "total topics is incorrect, fix: replace to 0... ";
      $Board->{'TOTAL_TOPICS'} = 0;
      $Fix++;
    }

    if (($Board->{'STATUS'} ne "30") && ($Board->{'STATUS'} ne "50") && ($Board->{'STATUS'} ne "70")) {
      print "status is incorrect, fix: replace to 30... ";
      $Board->{'STATUS'} = 30;
      $Fix++;
    }

    if (($Board->{'UB_CODES'} eq "0") && ($Board->{'HTML_TAGS'} eq "1")) {
      print "html tag boolean is incorrect, fix: replace to false... ";
      $Board->{'HTML_TAGS'} = 0;
      $Fix++;
    }

    my ($Date, $Time);

    if ($Board->{'LAST_POST_SECOND'}) {
      $Date = &Standard::ConvertEpochToDate( $Board->{'LAST_POST_SECOND'} );
      $Time = &Standard::ConvertEpochToTime( $Board->{'LAST_POST_SECOND'} );
      if (
        ( $Board->{'LAST_POST_DATE'} ne $Date ) ||
        ( $Board->{'LAST_POST_TIME'} ne $Time )
      ) {
        print "last post date/time is out dated, update: replace to $Date/$Time... ";
        $Board->{'LAST_POST_DATE'} = $Date;
        $Board->{'LAST_POST_TIME'} = $Time;
        $Fix++;
      }
    }

    if ($Board->{'LAST_UPDATE_SECOND'}) {
      $Date = &Standard::ConvertEpochToDate( $Board->{'LAST_UPDATE_SECOND'} );
      $Time = &Standard::ConvertEpochToTime( $Board->{'LAST_UPDATE_SECOND'} );
      if (
        ( $Board->{'LAST_UPDATE_DATE'} ne $Date ) ||
        ( $Board->{'LAST_UPDATE_TIME'} ne $Time )
      ) {
        print "last updated date/time is out dated, update: replace to $Date/$Time... ";
        $Board->{'LAST_UPDATE_DATE'} = $Date;
        $Board->{'LAST_UPDATE_TIME'} = $Time;
        $Fix++;
      }
    }

    if ($Fix > 0) {
      print "updating... ";
      if (
        &DB::Update (
          TABLE   => "Boards",
          VALUES  => $Board,
          KEY     => $Board->{'ID'}
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
      }
    } else {
      print "correct.\n";
    }

    $Stats{'CATEGORY'}->{ $Board->{'CATEGORY'} }{'BOARDS'}++;

    $Stats{'BOARD'}->{ $Board->{'ID'} }{'TOPICS'} = 0;
    $Stats{'BOARD'}->{ $Board->{'ID'} }{'POSTS'}  = 0;
  }

  $T1 = new Benchmark;
  print "- Done. Used".timestr( timediff($T1, $T0) )."\n\n";

  for (my $b = 0; $b <= $#{ $Boards }; $b++) {
    my $Board = $Boards->[$b];

    $T0 = new Benchmark;
    print "- Reading the board #id $Board->{'ID'} <".$Board->{'TITLE'}."> database...\n";
    (
      my ($Posts, $PostsIndex) = &DB::Query (
        TABLE   => "B".$Board->{'ID'}
      )
    ) || print "- Error: \"$DB::Error\".\n";

    for (my $p = 0; $p <= $#{ $Posts }; $p++) {
      my $Post = $Posts->[$p];
      my $Fix = 0;

      print "- - ".($p + 1).")Checking topic id# $Post->{'ID'} <".$Post->{'SUBJECT'}.">... ";

      if ($Post->{'USERNAME'} ne "") {
        unless ( exists ( $AccountsIndex->{ $Post->{'USERNAME'} } ) ) {
          print "username is not found, fix: replace to guest level...";
          $Post->{'USERNAME'}     = "";
          $Post->{'MEMBER_LEVEL'} = "";
          $Post->{'MEMBER_GROUP'} = "";
          $Fix++;
        } else {
          my $Account = $Accounts->[ $AccountsIndex->{ $Post->{'USERNAME'} } ];
          if ($Post->{'NICKNAME'} ne $Account->{'NICKNAME'}) {
            print "nickname is out dated, update: replace to $Account->{'NICKNAME'}... ";
            $Post->{'NICKNAME'} = $Account->{'NICKNAME'};
            $Fix++;
          }

          if ($Post->{'EMAIL'} ne $Account->{'EMAIL'}) {
            print "email is out dated, update: replace to $Account->{'EMAIL'}... ";
            $Post->{'EMAIL'} = $Account->{'EMAIL'};
            $Fix++;
          }

          if ($Post->{'MEMBER_LEVEL'} ne $Account->{'LEVEL_TEXT'}) {
            print "member level is out dated, update: replace to $Account->{'LEVEL_TEXT'}... ";
            $Post->{'MEMBER_LEVEL'} = $Account->{'LEVEL_TEXT'};
            $Fix++;
          }

          if ($Post->{'MEMBER_GROUP'} ne $Account->{'GROUP_TEXT'}) {
            print "member group is out dated, update: replace to $Account->{'GROUP_TEXT'}... ";
            $Post->{'MEMBER_GROUP'} = $Account->{'GROUP_TEXT'};
            $Fix++;
          }
        }
      }

      if ($Post->{'ROOT'} eq "0") {
        my @Replies = split (/,/, $Post->{'REPLIES'});
        shift @Replies;
        my @tmp;
        for (my $r = 0; $r <= $#Replies; $r++) {
          unless ( exists ( $PostsIndex->{ $Replies[$r] } ) ) {
            print "reply id# ".$Replies[$r]." is not found, fix: remove it... ";

            my $Counter = 0;
            while (unlink ($SYSTEM::DB_PATH."/B".$Board->{'ID'}."/".$Post->{'ID'}."-".++$Counter.".html")) {}

            print "removing the encoded source... ";
            unlink ($SYSTEM::DB_PATH."/B".$Board->{'ID'}."/".$Post->{'ID'}.".post") || print "error: \"$!\"... ";

            print "removing the source... ";
            unlink ($SYSTEM::DB_PATH."/B".$Board->{'ID'}."/".$Post->{'ID'}.".src" ) || print "error: \"$!\"... ";
            $Fix++;
          } else {
            push (@tmp, $Replies[$r]);
          }
        }
        @Replies = @tmp;

        if ($Post->{'NUMBER_REPLIES'} != scalar(@Replies)) {
          print "number of replies is incorrect, fix: replace to ".scalar(@Replies)."... ";
          $Post->{'NUMBER_REPLIES'} = scalar(@Replies);
          $Fix++;
        }

        $Stats{'CATEGORY'}->{ $Board->{'CATEGORY'} }{'TOPICS'}++;
        $Stats{'CATEGORY'}->{ $Board->{'CATEGORY'} }{'POSTS'}++;
        $Stats{'BOARD'}   ->{ $Board->{'ID'} }      {'TOPICS'}++;
        $Stats{'BOARD'}   ->{ $Board->{'ID'} }      {'POSTS'}++;
      } else {
        $Stats{'CATEGORY'}->{ $Board->{'CATEGORY'} }{'POSTS'}++;
        $Stats{'BOARD'}   ->{ $Board->{'ID'} }      {'POSTS'}++;
      }

      my ($Date, $Time);

      if ($Post->{'POST_SECOND'}) {
        $Date = &Standard::ConvertEpochToDate( $Post->{'POST_SECOND'} );
        $Time = &Standard::ConvertEpochToTime( $Post->{'POST_SECOND'} );
        if (
          ( $Post->{'POST_DATE'} ne $Date ) ||
          ( $Post->{'POST_TIME'} ne $Time )
        ) {
          print "post date/time is out dated, update: replace to $Date/$Time... ";
          $Post->{'POST_DATE'} = $Date;
          $Post->{'POST_TIME'} = $Time;
          $Fix++;
        }
      }

      if ($Post->{'LAST_POST_SECOND'}) {
        $Date = &Standard::ConvertEpochToDate( $Post->{'LAST_POST_SECOND'} );
        $Time = &Standard::ConvertEpochToTime( $Post->{'LAST_POST_SECOND'} );
        if (
          ( $Post->{'LAST_POST_DATE'} ne $Date ) ||
          ( $Post->{'LAST_POST_TIME'} ne $Time )
        ) {
          print "last post date/time is out dated, update: replace to $Date/$Time... ";
          $Post->{'LAST_POST_DATE'} = $Date;
          $Post->{'LAST_POST_TIME'} = $Time;
          $Fix++;
        }
      }

      if ($Post->{'LAST_UPDATE_SECOND'}) {
        $Date = &Standard::ConvertEpochToDate( $Post->{'LAST_UPDATE_SECOND'} );
        $Time = &Standard::ConvertEpochToTime( $Post->{'LAST_UPDATE_SECOND'} );
        if (
          ( $Post->{'LAST_UPDATE_DATE'} ne $Date ) ||
          ( $Post->{'LAST_UPDATE_TIME'} ne $Time )
        ) {
          print "last update date/time is out dated, update: replace to $Date/$Time... ";
          $Post->{'LAST_UPDATE_DATE'} = $Date;
          $Post->{'LAST_UPDATE_TIME'} = $Time;
          $Fix++;
        }
      }

      if ($Post->{'TOTAL_MODIFIED'} < 0) {
        print "total modified is incorrect, fix: replace to 0... ";
        $Post->{'TOTAL_MODIFIED'} = 0;
        $Fix++;
      }

      if ($Post->{'ATTACHMENT'}) {
        unless (-e $SYSTEM::UPLOAD_PATH."/Post-".$Board->{'ID'}."-".$Post->{'ID'}."-".$Post->{'ATTACHMENT'}) {
          print "attachment is not found, fix: remove it... ";
          $Post->{'ATTACHMENT'} = "";
          $Fix++;
        }
      }

      if ($Fix > 0) {
        print "updating... ";
        if (
          &DB::Update (
            TABLE   => "B".$Board->{'ID'},
            VALUES  => $Post,
            KEY     => $Post->{'ID'}
          )
        ) {
          print "success.\n";
        } else {
          print "error: \"$DB::Error\".\n";
        }
      } else {
        print "correct.\n";
      }
    }

    $T1 = new Benchmark;
    print "- Done. Used".timestr( timediff($T1, $T0) )."\n\n";
  }

  $T0 = new Benchmark;
  print "- Rebuilding all statistics data...\n";

  for (my $g = 0; $g <= $#{ $Groups }; $g++) {
    my $Group = $Groups->[$g];
    print "- - Checking member group id# $Group->{'ID'} total members statistic... ";
    if ($Group->{'TOTAL_MEMBERS'} != $Stats{'GROUP'}->{ $Group->{'ID'} }) {
      print "incorrect, fix: replace to ".$Stats{'GROUP'}->{ $Group->{'ID'} }."... ";
      $Group->{'TOTAL_MEMBERS'} = $Stats{'GROUP'}->{ $Group->{'ID'} };

      if (
        &DB::Update (
          TABLE   => "Groups",
          VALUES  => {
            TOTAL_MEMBERS => $Group->{'TOTAL_MEMBERS'}
          },
          KEY     => $Group->{'ID'}
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
      }
    } else {
      print "correct.\n";
    }
  }

  print "\n";

  for (my $c = 0; $c <= $#{ $Categories }; $c++) {
    my $Category = $Categories->[$c];
    my $Fix = 0;
    my $Statistic = $Stats{'CATEGORY'}->{ $Category->{'ID'} };

    print "- - Checking category id# $Category->{'ID'} total boards statistic... ";
    if ($Category->{'TOTAL_BOARDS'} != $Statistic->{'BOARDS'}) {
      print "incorrect, fix: replace to ".$Statistic->{'BOARDS'}."... ";
      $Category->{'TOTAL_BOARDS'} = $Statistic->{'BOARDS'};
      $Fix++;
    }

    print "total topics statistic... ";
    if ($Category->{'TOTAL_TOPICS'} != $Statistic->{'TOPICS'}) {
      print "incorrect, fix: replace to ".$Statistic->{'TOPICS'}."... ";
      $Category->{'TOTAL_TOPICS'} = $Statistic->{'TOPICS'};
      $Fix++;
    }

    print "total posts statistic... ";
    if ($Category->{'TOTAL_POSTS'} != $Statistic->{'POSTS'}) {
      print "incorrect, fix: replace to ".$Statistic->{'POSTS'}."... ";
      $Category->{'TOTAL_POSTS'} = $Statistic->{'POSTS'};
      $Fix++;
    }

    if ($Fix > 0) {
      print "updating... ";
      if (
        &DB::Update (
          TABLE   => "Categories",
          VALUES  => $Category,
          KEY     => $Category->{'ID'}
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
      }
    } else {
      print "correct.\n";
    }
  }

  print "\n";

  for (my $b = 0; $b <= $#{ $Boards }; $b++) {
    my $Board = $Boards->[$b];
    my $Fix = 0;
    my $Statistic = $Stats{'BOARD'}->{ $Board->{'ID'} };

    print "- - Checking board id# $Board->{'ID'} total topics statistic... ";
    if ($Board->{'TOTAL_TOPICS'} != $Statistic->{'TOPICS'}) {
      print "incorrect, fix: replace to ".$Statistic->{'TOPICS'}."... ";
      $Board->{'TOTAL_TOPICS'} = $Statistic->{'TOPICS'};
      $Fix++;
    }

    print "total posts statistic... ";
    if ($Board->{'TOTAL_POSTS'} != $Statistic->{'POSTS'}) {
      print "incorrect, fix: replace to ".$Statistic->{'POSTS'}."... ";
      $Board->{'TOTAL_POSTS'} = $Statistic->{'POSTS'};
      $Fix++;
    }

    if ($Fix > 0) {
      print "updating... ";
      if (
        &DB::Update (
          TABLE   => "Boards",
          VALUES  => $Board,
          KEY     => $Board->{'ID'}
        )
      ) {
        print "success.\n";
      } else {
        print "error: \"$DB::Error\".\n";
      }
    } else {
      print "correct.\n";
    }
  }

  $T1 = new Benchmark;
  print "- Done. Used".timestr( timediff($T1, $T0) )."\n\n";

  return 1;
}

###############################################################################
1; # end of Rebuild.lib.pl file
###############################################################################