###############################################################################
# Thread.lib.pl                                                               #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################
package Thread;

use strict;
use Benchmark;

&Standard::Require("UB_Thread.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Thread.src");
&Standard::Require("UB_Thread_Message.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Thread_Message.src");

&Standard::Require("UB_Print_Thread.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Print_Thread.src");
&Standard::Require("UB_Print_Thread_Message.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Print_Thread_Message.src");

sub All {
  my %in = (
    BOARD     => "",
    FROM      => "",
    TO        => "",
    DETAIL    => 1,
    @_
  );

  if ($in{'BOARD'} eq "") {
    return -1;
  }

  my ($Threads) = &DB::Query (
    TABLE   => "B".$in{'BOARD'},
    COLUMNS => [
      "ID"                ,
      "SUBJECT"           ,
      "ROOT"              ,
      "POST_SECOND"       ,
      "APROVED"
    ]                           ,
    WHERE   => {
      ROOT    => [0]            ,
      APROVED => [1]
    }                           ,
    MATCH   => "ALL"            ,
    SORT    => "POST_SECOND"    ,
    BY      => "Z-A"            ,
    TO      => ($in{'FROM'} ne "" and $in{'TO'} ne "") ? ($in{'FROM'}."-".$in{'TO'}) : ""
  );

  if ($in{'FROM'} ne "" and $in{'TO'} ne "") {
    print "- Begin to rebuild the topics from ".($in{'FROM'} + 1)." to ".($in{'TO'} + 1)." at board id# $in{'BOARD'}\n\n";
  } else {
    print "- Begin to rebuild the all topics at board id# $in{'BOARD'}\n\n";
  }

  my $T0   = new Benchmark;

  my $Counter = 0;
  my $Done    = 0;
  my $Fail    = 0;
  foreach my $Thread (@{ $Threads }) {
    my $T0   = new Benchmark;

    print "- - ".++$Counter.") Rebuilding the topic id# ".$Thread->{'ID'}." <".$Thread->{'SUBJECT'}.">... ";

    if (
      &Build (
        BOARD  => $in{'BOARD'}    ,
        THREAD => $Thread->{'ID'},
      )
    ) {
      my $T1   = new Benchmark;

      print "Done. Used ".timestr( timediff($T1, $T0) )."\n";
      $Done++;
    } else {
      print "Fail.\n";
      $Fail++;
    }
  }

  my $T1   = new Benchmark;

  print "\n- Done. Used ".timestr( timediff($T1, $T0) );

  return ($Counter, $Done, $Fail);
}

sub Build {
  my %in = (
    BOARD     => "",
    THREAD    => "",
    @_
  );

  if (($in{'BOARD'} eq "") || ($in{'THREAD'} eq "")) {
    return;
  }

  my ($Topic) = &DB::BinarySelect (
    TABLE   => "B".$in{'BOARD'},
    COLUMNS => [
      "ID"                ,
      "SUBJECT"           ,
      "USERNAME"          ,
      "NICKNAME"          ,
      "EMAIL"             ,
      "MEMBER_LEVEL"      ,
      "MEMBER_GROUP"      ,
      "NUMBER_REPLIES"    ,
      "POST_DATE"         ,
      "POST_TIME"         ,
      "LAST_UPDATE_DATE"  ,
      "LAST_UPDATE_TIME"  ,
      "TOTAL_MODIFIED"    ,
      "ROOT"              ,
      "HOST"              ,
      "IP"                ,
      "ATTACHMENT"        ,
      "SYMBOL"            ,
      "READ_ONLY"         ,
      "APROVED"
    ],
    KEY     => $in{'THREAD'}
  ) || return;

  if ($Topic->{'ROOT'} > 0) {
    $in{'THREAD'} = $Topic->{'ROOT'};
    ($Topic) = &DB::BinarySelect (
      TABLE   => "B".$in{'BOARD'},
      COLUMNS => [
        "ID"                ,
        "SUBJECT"           ,
        "USERNAME"          ,
        "NICKNAME"          ,
        "EMAIL"             ,
        "MEMBER_LEVEL"      ,
        "MEMBER_GROUP"      ,
        "NUMBER_REPLIES"    ,
        "POST_DATE"         ,
        "POST_TIME"         ,
        "LAST_UPDATE_DATE"  ,
        "LAST_UPDATE_TIME"  ,
        "TOTAL_MODIFIED"    ,
        "ROOT"              ,
        "HOST"              ,
        "IP"                ,
        "ATTACHMENT"        ,
        "SYMBOL"            ,
        "READ_ONLY"         ,
        "APROVED"
      ],
      KEY     => $Topic->{'ROOT'}
    ) || return;
  }

  return if ($Topic->{'APROVED'} != 1);

  my $TOPIC_SUBJECT   = $Topic->{'SUBJECT'};
  my $TOPIC_READ_ONLY = $Topic->{'READ_ONLY'};

  my $TOTAL_PAGES = &Standard::ceil( $Topic->{'NUMBER_REPLIES'} + 1, $GENERAL::POST_PER_PAGE);

  my ($Posts) = &DB::Query (
    TABLE   => "B".$in{'BOARD'},
    COLUMNS => [
      "ID"                ,
      "SUBJECT"           ,
      "USERNAME"          ,
      "NICKNAME"          ,
      "EMAIL"             ,
      "MEMBER_LEVEL"      ,
      "MEMBER_GROUP"      ,
      "ROOT"              ,
      "POST_SECOND"       ,
      "POST_DATE"         ,
      "POST_TIME"         ,
      "LAST_UPDATE_DATE"  ,
      "LAST_UPDATE_TIME"  ,
      "TOTAL_MODIFIED"    ,
      "HOST"              ,
      "IP"                ,
      "SYMBOL"            ,
      "ATTACHMENT"        ,
      "APROVED"
    ]                           ,
    WHERE   => {
      ROOT    => [$in{'THREAD'}],
      APROVED => [1]
    }                           ,
    MATCH   => "ALL"            ,
    SORT    => "POST_SECOND"    ,
    BY      => "A-Z"
  );

  unshift (@{ $Posts }, $Topic);

  my $P = 0;
  while ( unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$in{'THREAD'}."-".++$P.".topic") ) {
  }
  unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$in{'THREAD'}.".print");

  (
    my ($Board) = &DB::BinarySelect (
      TABLE   => "Boards",
      COLUMNS => ["ID", "TITLE", "CATEGORY", "STATUS", "MAJOR_MODERATOR", "MAJOR_MODERATOR_NICKNAME", "MAJOR_MODERATOR_EMAIL", "MINOR_MODERATORS"],
      KEY     => $in{'BOARD'}
    )
  ) || return;

  (
    my ($Category) = &DB::BinarySelect (
      TABLE   => "Categories",
      COLUMNS => ["ID", "TITLE"],
      KEY     => $Board->{'CATEGORY'}
    )
  ) || return;

  &Standard::FileOpen(*HEADER, "r", "$SYSTEM::DB_PATH/B".$Board->{'ID'}."/header.inc");
    my $HEADER = join ("", <HEADER>);
  close (HEADER);

  &Standard::FileOpen(*FOOTER, "r", "$SYSTEM::DB_PATH/B".$Board->{'ID'}."/footer.inc");
    my $FOOTER = join ("", <FOOTER>);
  close (FOOTER);

  my $PATH = $SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$in{'THREAD'};

  my $PRINT_THREAD;
  for (my $PAGE = 1; $PAGE <= $TOTAL_PAGES; $PAGE++) {
    my $THREAD = "";

    my $Start = (($PAGE - 1) * $GENERAL::POST_PER_PAGE);
    my $End   = ( $PAGE      * $GENERAL::POST_PER_PAGE);

    my $NumberPosts = scalar ( @$Posts );
    $End = $NumberPosts if ($End > $NumberPosts);

    for (my $PostIndex = $Start; $PostIndex < $End; $PostIndex++) {
      my $Post  = $Posts->[$PostIndex];
      $Post->{'SUBJECT'}  =~ s/\n/\\n/g;
      $Post->{'NICKNAME'} =~ s/\n/\\n/g;
      my %POST = (
        "POST.ID"               => $Post->{'ID'}              ,
        "POST.NUMBER"           => $PostIndex + 1             ,
        "POST.USERNAME"         => $Post->{'USERNAME'}        ,
        "POST.NICKNAME"         => $Post->{'NICKNAME'}        ,
        "POST.LEVEL"            => lc $Post->{'MEMBER_LEVEL'} ,
        "POST.GROUP"            => lc $Post->{'MEMBER_GROUP'} ,
        "POST.EMAIL"            => $Post->{'EMAIL'}           ,
        "POST.POST_DATE"        => $Post->{'POST_DATE'}       ,
        "POST.POST_TIME"        => $Post->{'POST_TIME'}       ,
        "POST.LAST_UPDATE_DATE" => $Post->{'LAST_UPDATE_DATE'},
        "POST.LAST_UPDATE_TIME" => $Post->{'LAST_UPDATE_TIME'},
        "POST.TOTAL_MODIFIED"   => $Post->{'TOTAL_MODIFIED'}  ,
        "POST.IP"               => $Post->{'IP'}              ,
        "POST.HOST"             => $Post->{'HOST'}            ,
        "POST.SUBJECT"          => $Post->{'SUBJECT'}         ,
        "POST.ATTACHMENT"       => $Post->{'ATTACHMENT'}      ,

        "POST.SYMBOL"           => ($UB::STYLE{'ICONS'}->{ $Post->{'SYMBOL'} }) ? $Post->{'SYMBOL'} : "",
        "POST.SYMBOL.URL"       => $SYSTEM::IMAGE_URL."/".$SYSTEM::DEFAULT_THEME."/".$UB::STYLE{'ICONS'}->{ $Post->{'SYMBOL'} },

        "THREAD.ID"             => $in{'THREAD'}              ,
        "BOARD.ID"              => $in{'BOARD'}
      );

      if (($POST{'POST.POST_DATE'} ne $POST{'POST.LAST_UPDATE_DATE'}) && ($POST{'POST.POST_TIME'} ne $POST{'POST.LAST_UPDATE_TIME'})) {
        $POST{'POST.UPDATED'} = "1";
      }

      &Standard::FileOpen(*POST, "r", $SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$Posts->[$PostIndex]{'ID'}.".post");
        $POST{'POST.MESSAGE'} = join ("", <POST>);
      close (POST);

      $THREAD       .= &Thread_Message::Parse(\%POST);
      $PRINT_THREAD .= &Print_Thread_Message::Parse(\%POST);
    }

    my %THREAD = (
      "THREAD.BODY"     => $THREAD            ,
      "THREAD.TOPIC"    => $TOPIC_SUBJECT     ,
      "THREAD.READ_ONLY"=> $TOPIC_READ_ONLY   ,
      "THREAD.ID"       => $in{'THREAD'}      ,
      "BOARD.ID"        => $in{'BOARD'}       ,
      "CATEGORY.ID"     => $Board->{'CATEGORY'}  ,

      "THREAD.SYMBOL"           => ($UB::STYLE{'ICONS'}->{ $Topic->{'SYMBOL'} }) ? $Topic->{'SYMBOL'} : "",
      "THREAD.SYMBOL.URL"       => $SYSTEM::IMAGE_URL."/".$SYSTEM::DEFAULT_THEME."/".$UB::STYLE{'ICONS'}->{ $Topic->{'SYMBOL'} },

      "SHOW.BOARDS.LIST"=> 1                     ,

      "ADDRESS"         => "<a href=\"$SYSTEM::UB_URL/UltraBoard.$SYSTEM::EXT?action=Category&CID=".$Category->{'ID'}."&SID=<\\~MEMBER.SESSION\\~>\" style=\"color:$UB::STYLE{'NORMAL_TEXT_COLOR'}\">".$Category->{'TITLE'}."</a>".
                           " / ".
                           "<a href=\"$SYSTEM::UB_URL/UltraBoard.$SYSTEM::EXT?action=Headlines&BID=".$Board->{'ID'}."&SID=<\\~MEMBER.SESSION\\~>\" style=\"color:$UB::STYLE{'NORMAL_TEXT_COLOR'}\">".$Board->{'TITLE'}."</a>",

      "BOARD.MAJOR_MODERATOR.USERNAME"  => $Board->{'MAJOR_MODERATOR'         },
      "BOARD.MAJOR_MODERATOR.NICKNAME"  => $Board->{'MAJOR_MODERATOR_NICKNAME'},
      "BOARD.MAJOR_MODERATOR.EMAIL"     => $Board->{'MAJOR_MODERATOR_EMAIL'   },

      "BOARD.HAVE.MAJOR_MODERATOR"      => ($Board->{'MAJOR_MODERATOR'})  ? 1 : 0,
      "BOARD.HAVE.MINOR_MODERATORS"     => ($Board->{'MINOR_MODERATORS'}) ? 1 : 0,

      "BOARD.HEADER"    => $HEADER                                            ,
      "BOARD.FOOTER"    => $FOOTER                                            ,

      "PAGE.CURRENT"    => $PAGE
    );

    $THREAD{'PAGE.PREVIOUS'} = $PAGE - 1 if $PAGE > 1;
    $THREAD{'PAGE.NEXT'}     = $PAGE + 1 if $PAGE < $TOTAL_PAGES;

    $THREAD{'PAGE.PREVIOUS2'} = $PAGE - 2 if $PAGE > 2;
    $THREAD{'PAGE.NEXT2'}     = $PAGE + 2 if $PAGE < ($TOTAL_PAGES - 1);

    &Compile( &Thread::Parse(\%THREAD), $PATH."-".$PAGE.".topic" );
  }

  my %THREAD = (
    "THREAD.BODY"     => $PRINT_THREAD,
    "THREAD.TOPIC"    => $TOPIC_SUBJECT,
    "THREAD.READ_ONLY"=> $TOPIC_READ_ONLY   ,
  );

  &Compile( &Print_Thread::Parse(\%THREAD), $PATH.".print" );

  return 1;
}

sub Compile {
  my ($TEXT, $PATH) = @_;

  $TEXT =~ s/\\/\\\\/g;
  $TEXT =~ s/\"/\\\"/g;
  $TEXT =~ s/\$/\\\$/g;
  $TEXT =~ s/\@/\\\@/g;
  $TEXT =~ s/\%/\\\%/g;
  $TEXT =~ s/\|/\&\#124\;/g;

  my $TAG_START = quotemeta("<\\\\~");
  my $TAG_END   = quotemeta("\\\\~>");

  $TEXT =~ s/\n/\\n\";\n\$HTML \.= \"/g;
  $TEXT =~ s/$TAG_START\s*([\w\.\-]+)\s*$TAG_END/\"\.(exists \$FORM->{\'$1\'} ? \$FORM->{\'$1\'} : "")\.\"/ig;

  while (
    $TEXT =~ s/$TAG_START\s*([\w-]+)\s*:\s*UNLESS\s*(\S+)\s+==\s+\\\"(\S+)\\\"\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*UNLESS\s*$TAG_END/\"\;\n\nif (\$FORM->{\'$2\'} ne \"$3\") {\n\n\$HTML \.= \"$4\";\n\n}\n\n\$HTML \.= \"/isg
  ) {

  }

  while (
    $TEXT =~ s/$TAG_START\s*([\w-]+)\s*:\s*IF\s*(\S+)\s+==\s+\\\"(\S+)\\\"\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*IF\s*$TAG_END/\"\;\n\nif (\$FORM->{\'$2\'} eq \"$3\") {\n\n\$HTML \.= \"$4\";\n\n}\n\n\$HTML \.= \"/isg
  ) {

  }

  while (
    $TEXT =~ s/$TAG_START\s*([\w-]+)\s*:\s*UNLESS\s*(\S+)\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*UNLESS\s*$TAG_END/\"\;\n\nif ( (! exists \$FORM->{\'$2\'}) || (\$FORM->{\'$2\'} eq \"0\") || (\$FORM->{\'$2\'} eq \"\") ) {\n\n\$HTML \.= \"$3\";\n\n}\n\n\$HTML \.= \"/isg
  ) {

  }

  while (
    $TEXT =~ s/$TAG_START\s*([\w-]+)\s*:\s*IF\s*(\S+)\s*$TAG_END(.+)$TAG_START\s*\1\s*:\s*END\s*IF\s*$TAG_END/\"\;\n\nif ( (exists \$FORM->{\'$2\'}) && ( (\$FORM->{\'$2\'} eq \"1\") || ((\$FORM->{\'$2\'} ne \"0\") && (\$FORM->{\'$2\'} ne \"\")) ) ) {\n\n\$HTML \.= \"$3\";\n\n}\n\n\$HTML \.= \"/isg
  ) {

  }

  $TEXT = "\$HTML \.= \"".$TEXT."\";\n";

  $TEXT =~ s/\$HTML \.= \"\\n\";\n//g;
  $TEXT =~ s/([^\\])\"\"\./$1/g;
  $TEXT =~ s/\.\"\"//g;

  &Standard::FileOpen(*TEXT, "w", $PATH);
    print TEXT "package Topic;\n\n";
    print TEXT "sub Parse {\n";
    print TEXT "my (\$FORM) = \@_;\n";
    print TEXT "my (\$HTML);\n\n";

    print TEXT $TEXT;

    print TEXT "\nreturn \$HTML;\n";
    print TEXT "}\n\n";

    print TEXT "1;";
  close (TEXT);
}

###############################################################################
1; # end of Thread.lib.pl file
###############################################################################