###############################################################################
# UB_DoActiveCode.pl                                                          #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################

package Source;

use strict;

sub DoActiveCode {

# =========================================================================== #
# Verify Variables

  # checking missing fields
  if ($UB::QUERY{'FORM_USERNAME'}    eq "") { &Error("MISS_USERNAME"    ) };
  if ($UB::QUERY{'FORM_EMAIL'}       eq "") { &Error("MISS_EMAIL"       ) };
  if ($UB::QUERY{'FORM_ANSWER'}      eq "") { &Error("MISS_ANSWER"      ) };
  if ($UB::QUERY{'FORM_ACTIVE_CODE'} eq "") { &Error("FORM_ACTIVE_CODE" ) };

  (
    my ($Password) = &DB::BinarySelect (
      TABLE   => "Password",
      KEY     => $UB::QUERY{'PID'}
    )
  ) || &HTML::Error (CODE => "FUNCTION_DISABLED");

  &Error("INVALID_EMAIL")    if ($Password->{'EMAIL'} ne $UB::QUERY{'FORM_EMAIL'});
  &Error("INVALID_ANSWER")   if ($Password->{'ANSWER'} ne $UB::QUERY{'FORM_ANSWER'});

  my $ACTIVE_CODE = &Standard::CryptPassword($UB::QUERY{'FORM_USERNAME'}, $UB::QUERY{'FORM_ACTIVE_CODE'});

  &Error("INVALID_ACTIVE_CODE") if ($ACTIVE_CODE ne $Password->{'ACTIVE_CODE'});

  if (
    ($UB::QUERY{'FORM_PASSWORD'} !~ /^[A-Za-z0-9]+$/) ||
    (length ($UB::QUERY{'FORM_PASSWORD'}) > 16)       ||
    (length ($UB::QUERY{'FORM_PASSWORD'}) <  4)
  ) {
    &Error("INVALID_PASSWORD");
  }

  if ($UB::QUERY{'FORM_PASSWORD'} ne $UB::QUERY{'FORM_VERIFY_PASSWORD'})  {
    &Error("UNMATCH_PASSWORD");
  }

  (
    my ($Account) = &DB::BinarySelect (
      TABLE   => "Accounts",
      COLUMNS => ["USERNAME", "EMAIL", "APROVED", "STATUS"],
      KEY     => lc ($Password->{'USERNAME'})
    )
  ) || &Error("ACCOUNT_NOT_FOUND");

  unless ($Account->{'APROVED'} == 1) {
    &HTML::Error (CODE => "ACCOUNT_NOT_APPROVED");
  }

  unless ($Account->{'STATUS'} == 50) {
    &HTML::Error (CODE => "ACCOUNT_INACTIVATE");
  }

  if ($Password->{'SECOND'} < (time - (60 * 60 * 48))) {
    &DB::Delete (
      TABLE   => "Password",
      KEY     => $UB::QUERY{'PID'}
    );
    &Error("CODE_EXPIRED");
  }

# =========================================================================== #
# Assign Variables

  my %FORM;

  $FORM{'USERNAME'} = lc $Password->{'USERNAME'};
  $FORM{'PASSWORD'} = &Standard::CryptPassword($FORM{'USERNAME'}, $UB::QUERY{'FORM_PASSWORD'});

  $FORM{'IP'}       = $ENV{'REMOTE_ADDR'};

  $FORM{'HOST'}     = $ENV{'REMOTE_HOST'};

# =========================================================================== #
# Update Member to Database

  ( # update member information from accounts database
    &DB::Update (
      TABLE   => "Accounts",
      VALUES  => {
        PASSWORD          => $FORM{'PASSWORD'},
        HOST              => $FORM{'HOST'}      ,
        IP                => $FORM{'IP'}
      },
      KEY     => $FORM{'USERNAME'}
    )
  ) || &HTML::Error(MESSAGE => $DB::Error);

  &DB::Delete (
    TABLE   => "Password",
    KEY     => $UB::QUERY{'PID'}
  );

  my %PASSWORD = (
    "PASSWORD.USERNAME"    => $FORM{'USERNAME'},
    "PASSWORD.EMAIL"       => $FORM{'EMAIL'}   ,
  );

  &Success(\%PASSWORD);
}

###############################################################################
# Error function
# - function of printing error message of activing code
sub Error {
  my ($code) = @_;

  require "$UB::SOURCES_PATH/UB_ActiveCode.pl";
  &Source::ActiveCode($code);

  exit;
}

###############################################################################
# Success function
# - function of printing success message of activing code
sub Success {
  my ($PASSWORD) = @_;

  &Standard::Require("UB_Active_Code_Success.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Active_Code_Success.src");

  &Standard::PrintHTMLHeader();
  print &Active_Code_Success::Parse($PASSWORD);
  exit;
}

###############################################################################
1; # end of UB_DoActiveCode.pl file
###############################################################################