###############################################################################
# UB_DoEmail.pl                                                               #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################

package Source;

use strict;

sub DoEmail {
  my $SendMail = 0;
  if ($SYSTEM::SENDMAIL == 1) {
    if ( ($SYSTEM::MAIL_SYSTEM eq "SMTP") && ($SYSTEM::SMTP_PATH ne "") ) {
      $SendMail = &Standard::Require("SMTP.lib.pl", "$UB::LIBRARIES_PATH/SMTP.lib.pl");
    } elsif ( ($SYSTEM::MAIL_SYSTEM eq "SENDMAIL") && ($SYSTEM::SENDMAIL_PATH ne "") ) {
      $SendMail = &Standard::Require("Sendmail.lib.pl", "$UB::LIBRARIES_PATH/Sendmail.lib.pl");
    }
  }

  &HTML::Error (CODE => "FUNCTION_DISABLED") unless ($SendMail);

  (
    my ($Account) = &DB::BinarySelect (
      TABLE   => "Accounts",
      KEY     => $UB::QUERY{'UID'}
    )
  ) || &HTML::Error(CODE => "ACCOUNT_NOT_FOUND");

  $Account = &Standard::UnReturn($Account);

# =========================================================================== #
# Verify Variables

  # checking missing fields
  if ($UB::QUERY{'FORM_SUBJECT'} eq "") { &Error("MISS_SUBJECT") };
  if ($UB::QUERY{'FORM_MESSAGE'} eq "") { &Error("MISS_MESSAGE") };

  if ($UB::QUERY{'FORM_YOUR_NICKNAME'} eq "") { &Error("MISS_YOUR_NICKNAME") };
  if ($UB::QUERY{'FORM_YOUR_EMAIL'}    eq "") { &Error("MISS_YOUR_EMAIL")    };

  # checking invalid email
  if (
    ($UB::QUERY{'FORM_YOUR_EMAIL'} !~ /[\.\w\-]+\@[\.\w\-]+\.[\.\w\-]+/) ||
    (length ($UB::QUERY{'FORM_YOUR_EMAIL'}) > 64)
  ) {
    &Error("INVALID_YOUR_EMAIL");
  }

  # checking length of fields
  if (length($UB::QUERY{'FORM_SUBJECT'}) > 64) {
    &Error("OVER_SUBJECT");
  }

  if (length($UB::QUERY{'FORM_MESSAGE'}) > 4096) {
    &Error("OVER_MESSAGE");
  }

# =========================================================================== #
# Assign Variables

  my %FORM;

  $FORM{'YOUR_NICKNAME'}    = $UB::QUERY{'FORM_YOUR_NICKNAME'};
  $FORM{'YOUR_NICKNAME'}    = &Standard::EncodeHTML($FORM{'YOUR_NICKNAME'});

  $FORM{'YOUR_EMAIL'}       = lc ($UB::QUERY{'FORM_YOUR_EMAIL'});

  $FORM{'FRIEND_NICKNAME'}  = $Account->{'NICKNAME'};
  $FORM{'FRIEND_NICKNAME'}  = &Standard::EncodeHTML($FORM{'FRIEND_NICKNAME'});

  $FORM{'FRIEND_EMAIL'}     = lc ($Account->{'EMAIL'});

  $FORM{'SUBJECT'}       = $UB::QUERY{'FORM_SUBJECT'};
  $FORM{'SUBJECT'}       = &Standard::EncodeHTML($FORM{'SUBJECT'});

  $FORM{'MESSAGE'}       = $UB::QUERY{'FORM_MESSAGE'};
  $FORM{'MESSAGE'}       = &Standard::EncodeHTML($FORM{'MESSAGE'});

  ($FORM{'SUBJECT'}, $FORM{'MESSAGE'}) = &Standard::WordFilter($FORM{'SUBJECT'}, $FORM{'MESSAGE'});

# =========================================================================== #
# Mailing to ...

  my %EMAIL = (
    "EMAIL.YOUR.NICKNAME"     => $FORM{'YOUR_NICKNAME'},
    "EMAIL.YOUR.EMAIL"        => $FORM{'YOUR_EMAIL'},
    "EMAIL.FRIEND.NICKNAME"   => $FORM{'FRIEND_NICKNAME'},
    "EMAIL.FRIEND.EMAIL"      => $FORM{'FRIEND_EMAIL'},
    "EMAIL.SUBJECT"           => $FORM{'SUBJECT'},
    "EMAIL.MESSAGE"           => $FORM{'MESSAGE'},

    &Standard::ExtraVariables()
  );

  $EMAIL{'EMAIL.MESSAGE'} .= "\n\n\n\n----------------------------------------\nPowered by UltraBoard 2000 <http://www.ultraboard.org/>\n";

  if (($SYSTEM::SENDMAIL == 1) && ($SendMail == 1)) {
    $EMAIL{'EMAIL.MESSAGE'} =~ s/\n\n/<p>/g;
    $EMAIL{'EMAIL.MESSAGE'} =~ s/\n/<br>/g;

    &Mail::Send (
      SUBJECT => $EMAIL{'EMAIL.SUBJECT'},
      MESSAGE => $EMAIL{'EMAIL.MESSAGE'},
      FROM    => "$FORM{'YOUR_NICKNAME'} <$FORM{'YOUR_EMAIL'}>",
      TO      => "$FORM{'FRIEND_NICKNAME'} <$FORM{'FRIEND_EMAIL'}>"
    );
  }

  &Success(\%EMAIL);
}

###############################################################################
# Error function
# - function of printing error message of forward topic
sub Error {
  my ($code) = @_;

  require "$UB::SOURCES_PATH/UB_Email.pl";
  &Source::Forward($code);

  exit;
}

###############################################################################
# Success function
# - function of printing success message of forward topic
sub Success {
  my ($POST) = @_;

  &Standard::Require("UB_Email_Success.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Email_Success.src");

  &Standard::PrintHTMLHeader();
  print &Email_Success::Parse($POST);
  exit;
}

###############################################################################
1; # end of UB_DoEmail.pl file
###############################################################################